from scipy.optimize.lbfgsb import fmin_l_bfgs_b
import openopt
from openopt.kernel.setDefaultIterFuncs import *
from openopt.kernel.ooMisc import WholeRepr2LinConst
from openopt.kernel.baseSolver import baseSolver
from numpy import isfinite

class scipy_lbfgsb(baseSolver):
    __name__ = 'scipy_lbfgsb'
    __license__ = "BSD"
    __authors__ = """Ciyou Zhu, Richard Byrd, and Jorge Nocedal <nocedal@ece.nwu.edu>,
    connected to scipy by David M. Cooke <cookedm@physics.mcmaster.ca> and Travis Oliphant,
    connected to openopt by Dmitrey"""
    __alg__ = "l-bfgs-b"
    __info__ = 'box-bounded limited-memory NLP solver, can handle lb<=x<=ub constraints, some lb-ub coords can be +/- inf'#TODO: add '__info__' field to other solvers
    __optionalDataThatCanBeHandled__ = ['lb', 'ub']
    __isIterPointAlwaysFeasible__ = lambda self, p: True

    def __init__(self):pass

    def __solver__(self, p):
        #WholeRepr2LinConst(p)#TODO: remove me

        bounds = []

        # don't work in Python ver < 2.5
        # BOUND = lambda x: x if isfinite(x) else None

        def BOUND(x):
            if isfinite(x): return x
            else: return None

        for i in range(p.n): bounds.append((BOUND(p.lb[i]), BOUND(p.ub[i])))

        xf, ff, d = fmin_l_bfgs_b(p.f, p.x0, fprime=p.df,
                  approx_grad=0,  bounds=bounds,
                  iprint=p.iprint, maxfun=p.maxFunEvals)

        if d['warnflag'] in (0, 2):
            # if 2 - some problems can be present, but final check from RunProbSolver will set negative istop if solution is unfeasible
            istop = SOLVED_WITH_UNIMPLEMENTED_OR_UNKNOWN_REASON
            if d['warnflag'] == 0: msg = 'converged'
        elif d['warnflag'] == 1:  istop = IS_MAX_FUN_EVALS_REACHED

        p.xk = p.xf = xf
        p.fk = p.ff = ff
        p.istop = istop
        p.iterfcn()




from baseProblem import MatrixProblem
#from numpy.linalg import norm
from numpy import vstack, isscalar

class EIG(MatrixProblem):
    probType = 'EIG'
    goal = 'all'
    allowedGoals = None
    showGoal = True
    expectedArgs = ['C']
    M = None
    _optionalData = ['M']
    xtol = 0.0
    FuncDesignerSign = 'C'
    N = 0
    
    #ftol = None
    def __init__(self, *args, **kwargs):
        MatrixProblem.__init__(self, *args, **kwargs)

        if self.goal == 'all':
            Name, name = 'all eigenvectors and eigenvalues', 'all'
            if not isinstance(self.C[0], dict):
                self.N = self.C.shape[0]
        else:
            assert type(self.goal) in (dict, tuple, list) and len(self.goal) == 1, \
            'EIG goal argument should be "all" or Python dict {goal_name: number_of_required_eigenvalues}'
            if type(self.goal) == dict:
                goal_name, N = list(self.goal.items())[0]
            else:
                goal_name, N = self.goal
            self.N = N
            name = ''.join(goal_name.lower().split())
            if name  in ('lm', 'largestmagnitude'):
                Name, name = 'largest magnitude', 'le'
            elif name in ('sm', 'smallestmagnitude'):
                Name, name = 'smallest magnitude', 'sm'
            elif name in ('lr', 'largestrealpart'):
                Name, name = 'largest real part', 'lr'
            elif name in ('sr', 'smallestrealpart'):
                Name, name = 'smallest real part', 'sr'
            elif name in ('li', 'largestimaginarypart'):
                Name, name = 'largest imaginary part', 'li'
            elif name in ('si', 'smallestimaginarypart'):
                Name, name = 'smallest imaginary part', 'si'
            elif name in ('la', 'largestamplitude'):
                Name, name = 'largestamplitude', 'la'
            elif name in ('sa', 'smallestamplitude'):
                Name, name = 'smallest amplitude', 'sa'
            elif name in ('be', 'bothendsofthespectrum'):
                Name, name = 'both ends of the spectrum', 'be'
        
        self.goal = Name
        self._goal = name
        #if not isinstance(self.C[0], dict):
            
    
    def solve(self, *args, **kw):
        C = self.C
        if type(C) in (tuple,  list) and isinstance(C[0], dict):
            from FuncDesigner import ootranslator
            K = set()
            N = 0
            varSizes = {}
            for d in C:
                K.update(d.keys())
                for key in d.keys():
                    if key in varSizes:
                        if varSizes[key] != d[key].shape[1]:
                            s = 'incorrect shape 2nd coordinate %d for variable %s, defined in other array as %d' %(d[key].shape[1], key.name, varSizes[key])
                            self.err(s)
                    else:
                        varSizes[key] = d[key].shape[1] if not isscalar(d[key]) else 1
                tmp = list(d.values())
                N += tmp[0].shape[0] if not isscalar(tmp[0]) else 1
            P = dict([(key, [0]*val) for key, val in varSizes.items()])
            T = ootranslator(P)
            C2 = vstack([T.pointDerivative2array(d) for d in C])
            self.C = C2
            if C2.shape != (N, N):
                self.err('square matrix of shape (%d,%d) expected, shape %s obtained instead' % (N, N, C2.shape))

        r = MatrixProblem.solve(self, *args, **kw)
        if type(C) in (tuple,  list) and isinstance(C[0], dict):
            r.eigenvectors = [T.vector2point(v) for v in self.eigenvectors.T]
        return r
        
    
    def objFunc(self, x):
        return 0
        #raise 'unimplemented yet'
        



from ooMisc import assignScript
from baseProblem import NonLinProblem
from numpy import asarray, ones, inf, array


class NLP(NonLinProblem):
    _optionalData = ['A', 'Aeq', 'b', 'beq', 'lb', 'ub', 'c', 'h']
    expectedArgs = ['f', 'x0']
    probType = 'NLP'
    allowedGoals = ['minimum', 'min', 'maximum', 'max']
    showGoal = True
    def __init__(self, *args, **kwargs):
        self.goal = 'minimum'
        NonLinProblem.__init__(self, *args, **kwargs)



from baseProblem import NonLinProblem
from numpy import sum, dot, asfarray
import NLP

class NLLSP(NonLinProblem):
    _optionalData = ['lb', 'ub', 'A', 'Aeq', 'b', 'beq', 'c', 'h']
    showGoal = False
    goal = 'minimum'
    probType = 'NLLSP'
    allowedGoals = ['minimum', 'min']
    isObjFunValueASingleNumber = False
    expectedArgs = ['f', 'x0']
    
    def __init__(self, *args, **kwargs):
        NonLinProblem.__init__(self, *args, **kwargs)

    def objFuncMultiple2Single(self, fv):
        return (fv ** 2).sum()

    def nllsp2nlp(self, solver, **solver_params):

        ff = lambda x: sum(asfarray(self.f(x))**2)
        p = NLP.NLP(ff, self.x0)
        #p = NLP.NLP(FF, self.x0)
        self.fill(p, sameConstraints=True)
        if self.userProvided.df:
            p.df = lambda x: dot(2*asfarray(self.f(x)), asfarray(self.df(x,useSparse=False)))
        p.f = ff



        def nllsp_iterfcn(*args,  **kwargs):
            p.primalIterFcn(*args,  **kwargs)
            p.xk = self.xk
            p.fk = p.f(p.xk)
            p.rk = self.rk
            # TODO: add nNaNs
            if self.istop != 0:
                p.istop = self.istop
            elif p.istop != 0:
                self.istop = p.istop

        p.primalIterFcn,  p.iterfcn = self.iterfcn, nllsp_iterfcn

        self.iprint = -1
        p.show = False

        r = p.solve(solver, **solver_params)
        #r.ff = ff(r.xf)

        return r



"""
OpenOpt GUI:
     function manage() usage example
"""

from openopt import NLP, manage
from numpy import cos, arange, ones, asarray, abs, zeros
N = 50
M = 5
p = NLP(lambda x: ((x-M)**2).sum(), cos(arange(N)))
p.lb, p.ub = -6*ones(N), 6*ones(N)
p.lb[3] = 5.5
p.ub[4] = 4.5
p.c = lambda x: [2* x[0] **4-32, x[1]**2+x[2]**2 - 8]
p.h = (lambda x: 1e1*(x[-1]-1)**4, lambda x: (x[-2]-1.5)**4)

"""
minTime is used here
for to provide enough time for user
to play with GUI
"""

minTime = 1.5 # sec
p.name = 'GUI_example'
p.minTime = minTime

"""
hence maxIter, maxFunEvals etc
will not trigger till minTime

only same iter point x_k-1=x_k
or some coords = nan
can stop calculations

other antistop criteria: minFunEvals, minIter, minCPUTime
however, some solvers cannot handle them
 """

# start=True means don't wait for user to press "Run"
r = manage(p,'ralg', plot=1, start=True)
"""
   or calling manage() as filed of p:
r = p.manage('algencan', plot=1)
"""
if r is not None:
    # r is None if user has pressed "Exit" button
    print 'objfunc val:', r.ff




from openopt.kernel.baseSolver import baseSolver
import asa as ASA
from openopt.kernel.setDefaultIterFuncs import SMALL_DELTA_X,  SMALL_DELTA_F

class asa(baseSolver):
    __name__ = 'asa'
    __license__ = "BSD"
    __authors__ = 'Lester Ingber, connected to Python by Robert Jordens, connected to OO by Dmitrey'
    __alg__ = "Adaptive Simulated Annealing"
    iterfcnConnected = False
    _requiresFiniteBoxBounds = True
    __homepage__ = 'http://pypi.python.org/pypi/pyasa'

    __info__ = "non-default parameters are not implemented yet"
    __optionalDataThatCanBeHandled__ = ['lb', 'ub']
    __isIterPointAlwaysFeasible__ = True#lambda self, p: p.__isNoMoreThanBoxBounded__()
    funcForIterFcnConnection = 'f'
    f_iter = 10000000

    def __init__(self):pass
    def __solver__(self, p):

        p.kernelIterFuncs.pop(SMALL_DELTA_X, None)
        p.kernelIterFuncs.pop(SMALL_DELTA_F, None)

        xf, ff, code, opt_struct = ASA.asa(p.f, p.x0, p.lb, p.ub, full_output=True)
        if p.istop == 0: p.istop = 1000





from numpy import tile, isnan, array, atleast_1d, asarray, logical_and, all, logical_or, any, nan, isinf, \
arange, vstack, inf, logical_not, take, abs, hstack, empty, \
prod, int16, int32, int64, log2, searchsorted, cumprod#where

# for PyPy
from openopt.kernel.nonOptMisc import where, isPyPy
from bisect import bisect, bisect_left, bisect_right

import numpy as np
from FuncDesigner import oopoint
from FuncDesigner.multiarray import multiarray
from interalgT import *

try:
    from bottleneck import nanargmin, nanmin, nanargmax, nanmax
except ImportError:
    from numpy import nanmin, nanargmin, nanargmax, nanmax
    
    
def nanargmin_axis(a, axis = None):
    if axis is None:
        ind = -1
        Min = inf
        for i, elem in enumerate(a.flat):
            if elem <= Min:
                ind = i
                Min = elem
        return np.nan if ind == -1 else ind
    
    assert axis in (0, 1), 'unimplemented yet'
    
    A = a if type(a) == np.ndarray and a.ndim > 1 else atleast_2d(a)
    
    inds = np.empty(A.shape[1-axis], int)
    inds.fill(-1)
    Mins = np.empty(A.shape[1-axis], A.dtype)
    if str(A.dtype).startswith('float'):
        Mins.fill(np.inf)
    elif axis == 0:
        Mins = A[:, 0].copy()
    else:
        Mins = A[0].copy()
    for i in range(A.shape[axis]):
        tmp = A[i] if axis == 0 else A[:, i]
        ind = tmp <= Mins
        Mins[ind] = tmp[ind]
        inds[ind] = i
    return inds

def nanargmax_axis(A, axis = None):
    if axis is None:
        ind = -1
        Max = -inf
        for i, elem in enumerate(A.flat):
            if elem >= Max:
                ind = i
                Max = elem
        return np.nan if ind == -1 else ind
    
    assert axis in (0, 1), 'unimplemented yet'
    
    inds = np.empty(A.shape[1-axis], int)
    inds.fill(-1)
    Maxs = np.empty(A.shape[1-axis], A.dtype)
    Maxs.fill(-np.inf)
    for i in range(A.shape[axis]):
        tmp = A[i] if axis == 0 else A[:, i]
        ind = tmp >= Maxs
        Maxs[ind] = tmp[ind]
        inds[ind] = i
    return inds

if isPyPy:
    nanargmin = nanargmin_axis
    nanargmax = nanargmax_axis
    
    
def func82(y, e, vv, f, dataType, p, Th = None, _s = None, indT = None):
    domain = oopoint([(v, (y[:, i], e[:, i])) for i, v in enumerate(vv)], skipArrayCast=True, isMultiPoint=True)
    domain.dictOfFixedFuncs = p.dictOfFixedFuncs
    domain._dictOfRedirectedFuncs = p._dictOfRedirectedFuncs
    domain._dictOfStochVars = p._dictOfStochVars
    domain.maxDistributionSize = p.maxDistributionSize
    domain.nPoints = y.shape[0]
    domain._p = p
    
    new = 0
    if new:
        r, r0, y, e, _s, indT = f.iqg(domain, dataType, UB = Th, y = y, e = e, 
                              _s = _s, indT = indT)
    else:
#        r, r0, _y, _e  = f.iqg(domain, dataType, UB = Th)
        r, r0  = f.iqg(domain, dataType, UB = Th)
    if r is None:
        return None, None, None, None, False, False, array([]), array([])
    
    o_l, o_u, a_l, a_u = [], [], [], []
    definiteRange = r0.definiteRange
    for v in vv:
        # TODO: rework and optimize it
        tmp = r.get(v, None)
        if tmp is not None:
            o_l.append(tmp[0].lb)
            o_u.append(tmp[1].lb)
            a_l.append(tmp[0].ub)
            a_u.append(tmp[1].ub)
            
            #TODO: mb don't compute definiteRange for modificationVar != None
#            definiteRange = logical_and(definiteRange, tmp[0].definiteRange)
#            definiteRange = logical_and(definiteRange, tmp[1].definiteRange)
        else:
            o_l.append(r0.lb)
            o_u.append(r0.lb)
            a_l.append(r0.ub)
            a_u.append(r0.ub)
            #definiteRange = logical_and(definiteRange, r0.definiteRange)
    o, a = hstack(o_l+o_u), hstack(a_l+a_u)    
    return y, e, o, a, definiteRange, domain.exactRange, _s, indT

def func10(y, e, vv):
    m, n = y.shape
    LB = [[] for i in range(n)]
    UB = [[] for i in range(n)]

    r4 = (y + e) / 2
    
    for i in range(n):
        t1, t2 = tile(y[:, i], 2*n), tile(e[:, i], 2*n)
        t1[(n+i)*m:(n+i+1)*m] = t2[i*m:(i+1)*m] = r4[:, i]
        
#        if vv[i].domain is bool:
#            indINQ = y[:, i] != e[:, i]
#            tmp = t1[(n+i)*m:(n+i+1)*m]
#            tmp[indINQ] = 1
#            tmp = t2[i*m:(i+1)*m]
#            tmp[indINQ] = 0
            
#        if vv[i].domain is bool:
#            t1[(n+i)*m:(n+i+1)*m] = 1
#            t2[i*m:(i+1)*m] = 0
#        else:
#            t1[(n+i)*m:(n+i+1)*m] = t2[i*m:(i+1)*m] = r4[:, i]
        
        LB[i], UB[i] = t1, t2
    
    domain = dict((v, (LB[i], UB[i])) for i, v in enumerate(vv))
    domain = oopoint(domain, skipArrayCast = True)
    domain.isMultiPoint = True
    domain.nPoints = 4 * m
    return domain

def func8(domain, func, dataType):
    TMP = func.interval(domain, dataType)
    #assert TMP.lb.dtype == dataType
    return asarray(TMP.lb, dtype=dataType), asarray(TMP.ub, dtype=dataType), TMP.definiteRange

def getr4Values(vv, y, e, tnlh, func, C, contol, dataType, p, fo = inf):
    #TODO: use fo with objective before constraints when objective is simpler than constraints
    m, n = y.shape
    wr4_0 = (y+e) / 2
    if tnlh is None:# or p.probType =='MOP':
        wr4 = wr4_0
    else:
        tnlh = tnlh.copy()
        tnlh[atleast_1d(tnlh<1e-300)] = 1e-300 # to prevent division by zero
        tnlh[atleast_1d(isnan(tnlh))] = inf #- check it!
        tnlh_l_inv, tnlh_u_inv = 1.0 / tnlh[:, :n], 1.0 / tnlh[:, n:]
        wr4 = (y * tnlh_l_inv + e * tnlh_u_inv) / (tnlh_l_inv + tnlh_u_inv)
        ind = tnlh_l_inv == tnlh_u_inv # especially important for tnlh_l_inv == tnlh_u_inv = 0
        wr4[ind] = (y[ind] + e[ind]) / 2
        wr4_1 = (wr4 + wr4_0)/2
        wr4 = vstack((wr4, wr4_0, wr4_1))
    adjustr4WithDiscreteVariables(wr4, p)
    if p._isStochastic:
        cs = dict((oovar, asarray(wr4[:, i], dataType).reshape(-1, 1).view(multiarray)) for i, oovar in enumerate(vv))
    else:
        cs = dict((oovar, asarray(wr4[:, i], dataType)) for i, oovar in enumerate(vv))
        
    cs = oopoint(cs, skipArrayCast = True, maxDistributionSize = p.maxDistributionSize)
    cs.isMultiPoint = True
    cs.dictOfFixedFuncs = p.dictOfFixedFuncs
    cs._dictOfRedirectedFuncs = p._dictOfRedirectedFuncs
    cs._dictOfStochVars = p._dictOfStochVars
    cs.nPoints = m
    
#    from FuncDesigner import FuncDesignerException
#    raise FuncDesignerException('asdf')
    
    m = wr4.shape[0]
    
    kw =  {'Vars': p.freeVars} if p.freeVars is None or (p.fixedVars is not None and len(p.freeVars) < len(p.fixedVars)) else {'fixedVars': p.fixedVars}
    kw['fixedVarsScheduleID'] = p._FDVarsID
    if len(C) != 0:
        r15 = empty(m, bool)
        r15.fill(True)
        for _c, f, r16, r17 in C:
            c = f(cs, **kw).flatten()
            ind = logical_and(c  >= r16, c <= r17) # here r16 and r17 are already shifted by required tolerance
            r15 = logical_and(r15, ind)
    else:
        r15 = True
        
    isMOP = p.probType =='MOP'
    if not any(r15):
        F = empty(m, dataType)
        F.fill(2**31-2 if dataType in (int16, int32, int64, int) else nan) 
        if isMOP:
            FF = [F for k in range(p.nf)]
    elif all(r15):
        if isMOP:
            FF = [fun(cs, **kw) for fun in func]
        else:
            F =atleast_1d(asarray(func(cs, **kw), dataType).view(np.ndarray)).flatten() \
            if func is not None else zeros(m) # func is None for SNLE
    else:
        #cs = dict([(oovar, (y[r15, i] + e[r15, i])/2) for i, oovar in enumerate(vv)])
        #cs = ooPoint(cs, skipArrayCast = True)
        #cs.isMultiPoint = True
        if isMOP:
            FF = []
            for fun in func:
                tmp = atleast_1d(asarray(fun(cs, **kw), dataType).view(np.ndarray)).flatten() 
                F = empty(m, dataType)
                F.fill(2**31-15 if dataType in (int16, int32, int64, int) else nan)
                F[r15] = tmp[r15]    
                FF.append(F)
        else:
            tmp = atleast_1d(asarray(func(cs, **kw), dataType).view(np.ndarray)).flatten() if func is not None \
            else zeros(m) # func is None for SNLE
            
            F = empty(m, dataType)
            F.fill(2**31-15 if dataType in (int16, int32, int64, int) else nan)
            F[r15] = tmp[r15]
    #TODO: reduce cs by ind before computations
    if isMOP:
        return array(FF).view(np.ndarray).T.reshape(m, len(func)).tolist(), wr4.tolist()
    else:
        return F, wr4


def adjustr4WithDiscreteVariables(wr4, p):
    # TODO: rework it for discrete variables
    for i in p._discreteVarsNumList:
        v = p._freeVarsList[i]
        
        if v.domain is bool or v.domain is 'bool':
            wr4[:, i] = where(wr4[:, i]<0.5, 0, 1)
        else:
            tmp = wr4[:, i]
            d = v.domain 
            if isPyPy:
                d2 = d.tolist()
                ind = atleast_1d([bisect_left(d2, val) for val in tmp])
                ind2 = atleast_1d([bisect_right(d2, val) for val in tmp])
            else:
                ind = searchsorted(d, tmp, side='left')
                ind2 = searchsorted(d, tmp, side='right')
            ind3 = where(ind!=ind2)[0]
            Tmp = tmp[ind3].copy()
            
            ind[ind==d.size] -= 1 # may be due to roundoff errors
            ind[ind==1] = 0
            ind2[ind2==d.size] -=1
            ind2[ind2==0] = 1 # may be due to roundoff errors
            tmp1 = asarray(d[ind], p.solver.dataType)
            tmp2 = asarray(d[ind2], p.solver.dataType)
            if Tmp.size!=0:
                if str(tmp1.dtype).startswith('int'):
                    Tmp = asarray(Tmp, p.solver.dataType)
                tmp2[ind3] = Tmp.copy()
                tmp1[ind3] = Tmp.copy()
            tmp = where(abs(tmp-tmp1)<abs(tmp-tmp2), tmp1, tmp2)
            #print max(abs(tmp-tmp1)), max(abs(tmp-tmp2))
            wr4[:, i] = tmp
    #print where(wr4==0)[0].size, where(wr4==1)[0].size

def r2(PointVals, PointCoords, dataType):
#    r23 = nanargmin(PointVals)
#    if isnan(r23):
    # for bottleneck v >= 0.8:
    r23 = 0 if np.all(isnan(PointVals)) else nanargmin(PointVals)
    # TODO: check it , maybe it can be improved
    #bestCenter = cs[r23]
    #r7 = array([(val[0][r23]+val[1][r23]) / 2 for val in domain.values()], dtype=dataType)
    #r8 = atleast_1d(r3)[r23] if not isnan(r23) else inf
    r7 = array(PointCoords[r23], dtype=dataType)
    r8 = atleast_1d(PointVals)[r23] 
    return r7, r8
    
def func3(an, maxActiveNodes, p):
#    dataHandling = p.solver.dataHandling
    minActiveNodes = p.solver.minActiveNodes
    m = len(an)
    if m <= maxActiveNodes:
        return an, []#array([], object)
    
    an1, _in = an[:maxActiveNodes], an[maxActiveNodes:]    
        
    if getattr(an1[0], 'tnlh_curr_best', None) is not None:
        #t0 = an1[0].tnlh_curr_best
        tnlh_curr_best_values = asarray([node.tnlh_curr_best for node in an1])
        
        #changes
        tmp = 2 ** (-tnlh_curr_best_values)
        Tmp = -cumprod(1.0-tmp)
        
        if isPyPy:
            ind2 = bisect(Tmp.tolist(), -0.05)
        else:
            ind2 = searchsorted(Tmp, -0.05)
        #changes end

        ind = ind2
        
        M = max((minActiveNodes, ind))
        
#        # IMPORTANT!
#        if M == 0: M = 1
        
        tmp1, tmp2 = an1[:M], an1[M:]
        an1 = tmp1
        _in = tmp2 + _in#hstack((tmp2, _in))
        
    # TODO: implement it for MOP as well
#    cond_min_uf = 0 and dataHandling == 'raw' and hasattr(an[0], 'key')            
#    
#    if cond_min_uf:
#        num_nlh = min((max((1, int(0.8*maxActiveNodes))), len(an1)))
#        num_uf = min((maxActiveNodes - num_nlh, int(maxActiveNodes/2)))
#        if num_uf < 15:
#            num_uf = 15
#        #an1, _in = an[:num_nlh], an[num_nlh:]    
#        Ind = np.argsort([node.key for node in _in])
#        
#        min_uf_nodes = _in[Ind[:num_uf]]
#        _in = _in[Ind[num_uf:]]
#        
#        an1 = an1 + min_uf_node#np.hstack((an1, min_uf_nodes))
        
    # changes end
    #print maxActiveNodes, len(an1), len(_in)
    
    return an1, _in

def func1(tnlhf, tnlhf_curr, residual, y, e, o, a, _s_prev, p, indT):
    m, n = y.shape
    w = arange(m)
    
    if p.probType == 'IP':
        oc_modL, oc_modU = o[:, :n], o[:, n:]
        ac_modL, ac_modU = a[:, :n], a[:, n:]
#            # TODO: handle nans
        mino = where(oc_modL < oc_modU, oc_modL, oc_modU)
        maxa = where(ac_modL < ac_modU, ac_modU, ac_modL)
    
        # Prev
        tmp = a[:, 0:n]-o[:, 0:n]+a[:, n:]-o[:, n:]
        t = nanargmin(tmp,1)
        d = 0.5*tmp[w, t]
        
        
        #New
#        tmp = a - o
#        t_ = nanargmin(tmp,1)
#        t = t_% n
#        d = tmp[w, t_]

#        ind = 2**(-n) >= (_s_prev - d)/asarray(d, 'float64')
        ind = 2**(1.0/n) * d >= _s_prev
        #new
#        ind = 2**(1.0/n) * d >= nanmax(maxa-mino, 1)
        
        #ind = 2**(-n) >= (_s_prev - _s)/asarray(_s, 'float64')
    
        #s2 = nanmin(maxa - mino, 1)
        #print (abs(s2/_s))
        
        # Prev
        _s = nanmin(maxa - mino, 1)
        
        # New
        #_s = nanmax(maxa - mino, 1)
#        _s = nanmax(a - o, 1)
        
        #ind = _s_prev  <= _s + ((2**-n / log(2)) if n > 15 else log2(1+2**-n)) 
        indD = logical_not(ind)
        indD = ind
        indD = None
        #print len(where(indD)[0]), len(where(logical_not(indD))[0])
#    elif p.probType == 'MOP':
#
#        raise 'unimplemented'
    else:
        if p.solver.dataHandling == 'sorted':
            _s = func13(o, a)
            
            t = nanargmin(a, 1) % n

            d = nanmax([a[w, t] - o[w, t], 
                    a[w, n+t] - o[w, n+t]], 0)
            
            ## !!!! Don't replace it by (_s_prev /d- 1) to omit rounding errors ###
            #ind = 2**(-n) >= (_s_prev - d)/asarray(d, 'float64')
            
            #NEW
            ind = d  >=  _s_prev / 2 ** (1.0e-12/n)
            #ind = d  >=  _s_prev / 2 ** (1.0/n)
            indD = empty(m, bool)
            indD.fill(True)
            #ind.fill(False)
            ###################################################
        elif p.solver.dataHandling == 'raw':
            if p.probType == 'MOP':
                t = p._t[:m]
                p._t = p._t[m:]
                d = _s = p.__s[:m]
                p.__s = p.__s[m:]
            else:
#                tnlh_1, tnlh_2 = tnlhf[:, 0:n], tnlhf[:, n:]
#                TNHLF_min =  where(logical_or(tnlh_1 > tnlh_2, isnan(tnlh_1)), tnlh_2, tnlh_1)
#               # Set _s
#                _s = nanmin(TNHLF_min, 1)
                T = tnlhf_curr
                tnlh_curr_1, tnlh_curr_2 = T[:, 0:n], T[:, n:]
                TNHL_curr_min =  where(logical_or(tnlh_curr_1 < tnlh_curr_2, isnan(tnlh_curr_2)), tnlh_curr_1, tnlh_curr_2)
                t = nanargmin(TNHL_curr_min, 1)
                T = tnlhf
                d = nanmin(vstack(([T[w, t], T[w, n+t]])), 0)
                _s = d

            #OLD
            #!#!#!#! Don't replace it by _s_prev - d <= ... to omit inf-inf = nan !#!#!#
            #ind = _s_prev  <= d + ((2**-n / log(2)) if n > 15 else log2(1+2**-n)) 
            #ind = _s_prev - d <= ((2**-n / log(2)) if n > 15 else log2(1+2**-n)) 
            
            #NEW
            if any(_s_prev < d):
                pass
            ind = _s_prev  <= d + 1.0/n
#            T = TNHL_curr_min
            #ind2 = nanmin(TNHL_curr_min, 0)
            
            indQ = d >= _s_prev - 1.0/n 
            #indQ = logical_and(indQ, False)
            indD = logical_or(indQ, logical_not(indT))
#            print '------'
#            print indQ[:10]
#            print indD[:10]
#            print _s_prev[:2], d[:2]
            #print len(where(indD)[0]), len(where(indQ)[0]), len(where(indT)[0])
            #print _s_prev - d
            ###################################################
            #d = ((tnlh[w, t]* tnlh[w, n+t])**0.5)
        else:
            assert 0

    if any(ind):
        r10 = where(ind)[0]
        #print('r10:', r10)
#        print _s_prev
#        print ((_s_prev -d)*n)[r10]
#        print('ind length: %d' % len(where(ind)[0]))
#        print where(ind)[0].size
        #bs = e[ind] - y[ind]
        #t[ind] = nanargmax(bs, 1) # ordinary numpy.argmax can be used as well
        bs = e[r10] - y[r10]
        t[r10] = nanargmax(bs, 1) # ordinary numpy.argmax can be used as well

    return t, _s, indD
    
def func13(o, a): 
    m, n = o.shape
    n /= 2
#    if case == 1:
#        U1, U2 = a[:, :n].copy(), a[:, n:] 
#        #TODO: mb use nanmax(concatenate((U1,U2),3),3) instead?
#        U1 = where(logical_or(U1<U2, isnan(U1)),  U2, U1)
#        return nanmin(U1, 1)
        
    L1, L2, U1, U2 = o[:, :n], o[:, n:], a[:, :n], a[:, n:] 
#    if case == 2:
    U = where(logical_or(U1<U2, isnan(U1)),  U2, U1)
    L = where(logical_or(L2<L1, isnan(L1)), L2, L1)
    return nanmax(U-L, 1)

def func2(y, e, t, vv):#, tnlhf_curr):
    #assert y.size != 0
    if y.size == 0: return y.copy(), e.copy() # especially for PyPy
    new_y, new_e = y.copy(), e.copy()
    m, n = y.shape
    w = arange(m)
    
    #!!!! TODO: rework it for discrete problems    
    th = (new_y[w, t] + new_e[w, t]) / 2
    
    ### !!!!!!!!!!!!!!!!!!!!!
    # TODO: rework it, don't recalculate each time
    DiscreteVars =  [v.domain is not None for v in vv] 
    BoolVars = [(v.domain is bool or v.domain is 'bool') for v in vv]

    indDiscrete = where(DiscreteVars)[0]
    discreteCoordsSet = set(indDiscrete)
    
    if 1 and all(BoolVars):
        # TODO: use it where y != e
        ind = y[w, t] != e[w, t]
        if any(ind):
            w2, t2 = w[ind], t[ind]
            new_y[w2, t2] = 1
            new_e[w2, t2] = 0
    
    else:
        new_y[w, t] = th
        new_e[w, t] = th
        
        # TODO: mb rework it
        for ind, coord in enumerate(t):
            if coord in discreteCoordsSet:
                #d = vv[coord]
                # = (0, 1) if d is bool or d is 'bool' else\
                m1, m2 = \
                splitDomainForDiscreteVariable(array([y[ind, coord]]), array([e[ind, coord]]), vv[coord])
                
                new_y[ind, coord] = m2
                new_e[ind, coord] = m1

    new_y = vstack((y, new_y))
    new_e = vstack((new_e, e))

    
#    if tnlhf_curr is not None:
#        tnlhf_curr_local = hstack((tnlhf_curr[w, t], tnlhf_curr[w, n+t]))
#    else:
#        tnlhf_curr_local = None
    return new_y, new_e#, tnlhf_curr_local


def func12(an, maxActiveNodes, p, Solutions, vv, varTols, fo):
    solutions, r6 = Solutions.solutions, Solutions.coords
    if len(an) == 0:
        return array([]), array([]), [], array([])
    _in = an
    
    if r6.size != 0:
        r11, r12 = r6 - varTols, r6 + varTols
    y, e, S = [], [], []
    #Tnlhf_curr_local = []
    n = p.n
    N = 0
    maxSolutions = p.maxSolutions
    
#    new = 1
#    # new
#    if new and p.probType in ('MOP', 'SNLE', 'GLP', 'NLP', 'MINLP') and p.maxSolutions == 1:
#        
#        
#        return y, e, _in, _s
        
    
    while True:
        an1Candidates, _in = func3(_in, maxActiveNodes, p)

        #print nanmax(2**(-an1Candidates[0].tnlh_curr)) ,  nanmax(2**(-an1Candidates[-1].tnlh_curr))
        yc, ec, oc, ac, SIc = asarray([t.y for t in an1Candidates]), \
        asarray([t.e for t in an1Candidates]), \
        asarray([t.o for t in an1Candidates]), \
        asarray([t.a for t in an1Candidates]), \
        asarray([t._s for t in an1Candidates])

    
        if p.probType == 'MOP':
            tnlhf_curr = asarray([t.tnlh_all for t in an1Candidates])
            tnlhf = None        
        elif p.solver.dataHandling == 'raw':
            tnlhf = asarray([t.tnlhf for t in an1Candidates]) 
            tnlhf_curr = asarray([t.tnlh_curr for t in an1Candidates]) 
        else:
            tnlhf, tnlhf_curr = None, None
        
        
        if p.probType != 'IP': 
            #nlhc = asarray([t.nlhc for t in an1Candidates])
            
            #residual = asarray([t.residual for t in an1Candidates]) 
            residual = None
            
            indT = func4(p, yc, ec, oc, ac, fo, tnlhf_curr)

            if an1Candidates[0].indtc is not None:
                indtc = asarray([t.indtc for t in an1Candidates])
                indT = logical_or(indT, indtc)
        else:
            residual = None
            indT = None
        t, _s, indD = func1(tnlhf, tnlhf_curr, residual, yc, ec, oc, ac, SIc, p, indT)
        
        NewD = 1
        
        if NewD and indD is not None: 
            s4d = _s[indD]
            sf = _s[logical_not(indD)]

            _s = hstack((s4d, s4d, sf))
            yf, ef = yc[logical_not(indD)], ec[logical_not(indD)]
            yc, ec = yc[indD], ec[indD]
            t = t[indD]
        else:
            _s = tile(_s, 2)

        #yc, ec, tnlhf_curr_local = func2(yc, ec, t, vv, tnlhf_curr)
        yc, ec = func2(yc, ec, t, vv)

        if NewD and indD is not None:
            yc = vstack((yc, yf))
            ec = vstack((ec, ef))
            
        if maxSolutions == 1 or len(solutions) == 0: 
            #y, e, Tnlhf_curr_local = yc, ec, tnlhf_curr_local
            y, e = yc, ec
            break
        
        # TODO: change cycle variable if len(solutions) >> maxActiveNodes
        for i in range(len(solutions)):
            ind = logical_and(all(yc >= r11[i], 1), all(ec <= r12[i], 1))
            if any(ind):
                j = where(logical_not(ind))[0]
                lj = j.size
                yc = take(yc, j, axis=0, out=yc[:lj])
                ec = take(ec, j, axis=0, out=ec[:lj])
                _s = _s[j]
#                if tnlhf_curr_local is not None:
#                    tnlhf_curr_local = tnlhf_curr_local[j]
        y.append(yc)
        e.append(ec)
        S.append(_s)
        #Tnlhf_curr_local.append(tnlhf_curr_local)
        N += yc.shape[0]
        if len(_in) == 0 or N >= maxActiveNodes: 
            y, e, _s = vstack(y), vstack(e), hstack(S)
            #Tnlhf_curr_local = hstack(Tnlhf_curr_local)
            break
            
#    if Tnlhf_curr_local is not None and len(Tnlhf_curr_local) != 0 and Tnlhf_curr_local[0] is not None:
#        #print len(where(isfinite(Tnlhf_curr_local))[0]), Tnlhf_curr_local.size
#        pass

#    print 'y_prev:', y
#    print 'e_prev:', e
#    print '_s_prev:', hstack(_s)
    #print 'prev!', y, e, _s
    
#    from numpy import array_equal
#    if not array_equal(NEW_lx.sort(), y.sort()):
#        pass
#    if not array_equal(NEW_ux.sort(), e.sort()):
#        pass
#    if not array_equal(NEW__s.sort(), _s.sort()):
#        pass
        
        #, NEW_ux, NEW__in, NEW__s
    return y, e, _in, _s

Fields = ['key', 'y', 'e', 'nlhf','nlhc', 'indtc','residual','o', 'a', '_s']
MOP_Fields = ['y', 'e', 'nlhf','nlhc', 'indtc','residual','o', 'a', '_s']

#FuncValFields = ['key', 'y', 'e', 'nlhf','nlhc', 'o', 'a', '_s','r18', 'r19']
IP_fields = ['key', 'minres', 'minres_ind', 'complementary_minres', 'y', 'e', 'o', 'a', '_s','F', 'volume', 'volumeResidual']

def func11(y, e, nlhc, indTC, residual, o, a, _s, p): 
    m, n = y.shape
    if p.probType == "IP":
        w = arange(m)
        # TODO: omit recalculation from func1
        ind = nanargmin(a[:, 0:n] - o[:, 0:n] + a[:, n:] - o[:, n:], 1)
        sup_inf_diff = 0.5*(a[w, ind] - o[w, ind] + a[w, n+ind] - o[w, n+ind])
        diffao = a - o
        minres_ind = nanargmin(diffao, 1) 
        minres = diffao[w, minres_ind]
        complementary_minres = diffao[w, where(minres_ind<n, minres_ind+n, minres_ind-n)]
        volume = prod(e-y, 1)
        volumeResidual = volume * sup_inf_diff
        F = 0.25 * (a[w, ind] + o[w, ind] + a[w, n+ind] + o[w, n+ind])
        return [si(IP_fields, sup_inf_diff[i], minres[i], minres_ind[i], complementary_minres[i], y[i], e[i], o[i], a[i], _s[i], F[i], volume[i], volumeResidual[i]) for i in range(m)]
        
    else:
        residual = None
        
        isSNLE = p.probType in ('SNLE', 'NLSP')
        if 1 or not isSNLE:
            o, a = asarray(o), asarray(a)
            a[a==inf] = 1e300
            o[o==-inf] = -1e300
            tmp = a - o
            tmp[tmp<1e-300] = 1e-300
#            ind_uf_inf = where(a==inf)[0]
#            if ind_uf_inf.size:
#                Tmp = o[ind_uf_inf]
#                Tmp[Tmp==-inf] = -1e100
#                M = nanmax(abs(Tmp))
#                if M is nan or M == 0.0: 
#                    M = 1.0
#                tmp[ind_uf_inf] = 1e200 * (1.0 + Tmp/M)
            nlhf = log2(tmp)#-log2(p.fTol)
            
#        nlhf[a==inf] = 1e300# to make it not inf and nan
#        nlhf[o==-inf] = 1e300# to make it not inf and nan
        
        if p.probType == "MOP":
            if nlhf.ndim == 3: # in MOP
                nlhf = nlhf.sum(axis=1)
            else:
                assert 0, 'bug in interalg'
            # make correct o,a wrt each target
            return [si(MOP_Fields, y[i], e[i], nlhf[i], 
                          nlhc[i] if nlhc is not None else None, 
                          indTC[i] if indTC is not None else None, 
                          residual[i] if residual is not None else None, 
                          [o[i][k] for k in range(p.nf)], [a[i][k] for k in range(p.nf)], 
                          _s[i]) for i in range(m)]
        else:
            assert p.probType in ('GLP', 'NLP', 'NSP', 'SNLE', 'NLSP', 'MINLP', 'QP', 'LP', 'MILP')
            
            if 0 and isSNLE:
                nlhf = Tmp = o = a = [None]*m
            else:
                s, q = o[:, 0:n], o[:, n:2*n]
                Tmp = nanmax(where(q<s, q, s), 1)
                
                nlhf[logical_and(isinf(a), isinf(nlhf))] = 1e300
            
#            residual = None

            return [si(Fields, Tmp[i], y[i], e[i], nlhf[i], 
                          nlhc[i] if nlhc is not None else None, 
                          indTC[i] if indTC is not None else None, 
                          residual[i] if residual is not None else None, 
                          o[i], a[i], _s[i]) for i in range(m)]
    
#    else:
#        r18, r19 = r3[:, :n], r3[:, n:]
#        return [si(FuncValFields, Tmp[i], y[i], e[i], nlhf[i], nlhc[i] if nlhc is not None else None, o[i], a[i], _s[i], r18[i], r19[i]) for i in range(m)]

class si:
    def __init__(self, fields, *args, **kwargs):
        for i in range(len(fields)):
            setattr(self, fields[i], args[i])
    

#        new = 0
#        nn = 0
#if new and p.probType in ('MOP', 'SNLE', 'NLSP','GLP', 'NLP', 'MINLP') and p.maxSolutions == 1:
#            arr = tnlhf_curr if p.solver.dataHandling == 'raw' else oc
#            M = arr.shape[0]
#            w = arange(M)
#            Midles = 0.5*(yc[w, t] + ec[w, t])
#            arr_1, arr2 = arr[w, t], arr[w, n+t]
#            Arr = hstack((arr_1, arr2))
#            ind = np.argsort(Arr)
#            Ind = set(ind[:maxActiveNodes])
#            tag_all, tag_1, tag_2 = [], [], []
#            sn = []
#            
#            # TODO: get rid of the cycles
#            for i in range(M):
#                cond1, cond2 = i in Ind, (i+M) in Ind
#                if cond1:
#                    if cond2:
#                        tag_all.append(i)
#                    else:
#                        tag_1.append(i)
#                else:
#                    if cond2:
#                        tag_2.append(i)
#                    else:
#                        sn.append(an1Candidates[i])
#
#            list_lx, list_ux = [], []
#            
#            _s_new = []
#            updateTC = an1Candidates[0].indtc is not None
#            isRaw = p.solver.dataHandling == 'raw'
#            for i in tag_1:
#                node = an1Candidates[i]
#                I = t[i]
##                if node.o[n+I] >= node.o[I]:
##                    print '1'
##                else:
##                    print i, I, node.o[n+I] ,  node.o[I], node.key, node.a[n+I] ,  node.a[I], node.nlhc[n+I], node.nlhc[I]
#                node.key = node.o[n+I]
#                node._s = _s[i]
#                
#                if isRaw:
#                    node.tnlh_curr[I] = node.tnlh_curr[n+I]
#                    node.tnlh_curr_best = nanmin(node.tnlh_curr)
#                
#                #assert node.o[n+I] >= node.o[I]
#                #lx, ux = node.y, node.e
#                lx, ux = yc[i], ec[i]
#                if nn:
#                    #node.o[I], node.a[I] = node.o[n+I], node.a[n+I]
#                    node.o[I], node.a[I] = node.o[n+I], node.a[n+I]
#                    node.o[node.o<node.o[n+I]], node.a[node.a>node.a[n+I]] = node.o[n+I], node.a[n+I]
#                else:
#                    node.o[n+I], node.a[n+I] = node.o[I], node.a[I]
#                    node.o[node.o<node.o[I]], node.a[node.a>node.a[I]] = node.o[I], node.a[I]
#
##                if p.solver.dataHandling == 'raw':
#                for Attr in ('nlhf','nlhc', 'tnlhf', 'tnlh_curr', 'tnlh_all'):
#                    r = getattr(node, Attr, None)
#                    if r is not None:
#                        if nn: r[I] = r[n+I]
#                        else: 
#                            r[n+I] = r[I]
#
#                mx = ux.copy()
#                mx[I] = Midles[i]#0.5*(lx[I] + ux[I])
#                list_lx.append(lx)
#                list_ux.append(mx)
#                node.y = lx.copy()
#                node.y[I] = Midles[i]#0.5*(lx[I] + ux[I])
#                if updateTC: 
#                    node.indtc = True
#                
#                _s_new.append(node._s)
#                sn.append(node)
#            
#            for i in tag_2:
#                node = an1Candidates[i]
#                I = t[i]
#                node.key = node.o[I]
#
#                
#                node._s = _s[i]
#                
#                # for raw only
#                if isRaw:
#                    node.tnlh_curr[n+I] = node.tnlh_curr[I]
#                    node.tnlh_curr_best = nanmin(node.tnlh_curr)
#                
#                #assert node.o[I] >= node.o[n+I]
#                #lx, ux = node.y, node.e
#                lx, ux = yc[i], ec[i]
#
#                if nn:
#                    node.o[n+I], node.a[n+I] = node.o[I], node.a[I]
#                    node.o[node.o<node.o[I]], node.a[node.a>node.a[I]] = node.o[I], node.a[I]
#                else:
#                    node.o[I], node.a[I] = node.o[n+I], node.a[n+I]
#                    node.o[node.o<node.o[n+I]], node.a[node.a>node.a[n+I]] = node.o[n+I], node.a[n+I]
#                for Attr in ('nlhf','nlhc', 'tnlhf', 'tnlh_curr', 'tnlh_all'):
#                    r = getattr(node, Attr, None)
#                    if r is not None:
#                        if nn: r[n+I] = r[I]
#                        else: 
#                            r[I] = r[n+I]
#
#                mx = lx.copy()
#                mx[I] = Midles[i]#0.5*(lx[I] + ux[I])
#                list_lx.append(mx)
#                list_ux.append(ux)
#                node.e = ux.copy()
#                node.e[I] = Midles[i]#0.5*(lx[I] + ux[I])
#                if updateTC: 
#                    node.indtc = True
#
#                _s_new.append(node._s)
#                sn.append(node)
#            
#            for i in tag_all:
#                node = an1Candidates[i]
#                I = t[i]
#                
#                #lx, ux = node.y, node.e
#                lx, ux = yc[i], ec[i]
#                mx = ux.copy()
#                mx[I] = Midles[i]#0.5 * (lx[I] + ux[I])
#                
#                list_lx.append(lx)
#                list_ux.append(mx)
#                
#                mx = lx.copy()
#                mx[I] = Midles[i]#0.5 * (lx[I] + ux[I])
#                #mx[n+ t] = 0.5 * (lx[n + t] + ux[n + t])
#                list_lx.append(mx)
#                list_ux.append(ux)
#                
#                #_s_new += [_s[i]] * 2
#                _s_new.append(_s[i])
#                _s_new.append(_s[i])
#                
##            print 'y_new:', vstack(list_lx)
##            print 'e_new:', vstack(list_ux)
##            print '_s_new:', hstack(_s)
#            _in = sn + _in.tolist()
#            if p.solver.dataHandling == 'sorted':
#                _in.sort(key = lambda obj: obj.key)
#            else:
#                #pass
#                _in.sort(key = lambda obj: obj.tnlh_curr_best)
##            print 'tag 1:', len(tag_1), 'tag 2:', len(tag_2), 'tag all:', len(tag_all)
##            print 'lx:', list_lx
##            print 'sn lx:', [node.y for node in sn]
##            print 'ux:', list_ux
##            print 'sn ux:', [node.e for node in sn]
##            print '-'*10
#            #print '!', vstack(list_lx), vstack(list_ux), hstack(_s_new)
#            NEW_lx, NEW_ux, NEW__in, NEW__s = \
#            vstack(list_lx), vstack(list_ux), _in, hstack(_s_new)
#            return NEW_lx, NEW_ux, NEW__in, NEW__s



from openopt.kernel.baseSolver import baseSolver
from openopt.kernel.ooMisc import isSolved
from openopt.kernel.setDefaultIterFuncs import IS_NAN_IN_X, SMALL_DELTA_X, SMALL_DELTA_F
#from numpy import asarray, inf, ones, nan
from numpy import *
#import openopt
from openopt import NLP, OpenOptException

class branb(baseSolver):
    __name__ = 'branb'
    __license__ = "BSD"
    __authors__ = "Ingar Solberg, Institutt for teknisk kybernetikk, Norges Tekniske Hrgskole, Norway, translated to Python by Dmitrey"
    __homepage__ = ''
    __alg__ = "branch-and-cut (currently the implementation is quite primitive)"
    __info__ = ''
    __optionalDataThatCanBeHandled__ = ['A', 'Aeq', 'b', 'beq', 'lb', 'ub', 'discreteVars', 'c', 'h']
    iterfcnConnected = True
    __isIterPointAlwaysFeasible__ = lambda self, p: True
    #eps = 1e-7
    nlpSolver = None

    
    def __init__(self): pass

    def __solver__(self, p):
        if self.nlpSolver is None: p.err('you should explicitely provide parameter nlpSolver (name of NLP solver to use for NLP subproblems)')
        
        # TODO: check it
        # it can be removed in other place during prob preparation
        for key in [IS_NAN_IN_X, SMALL_DELTA_X, SMALL_DELTA_F]:
            if key in p.kernelIterFuncs.keys():
                p.kernelIterFuncs.pop(key)
        
        p.nlpSolver = self.nlpSolver
        startPoint = p.point(p.x0)
        startPoint._f = inf
        fPoint = fminconset(p, startPoint, p)
        p.iterfcn(fPoint)
        p.istop = 1000
        
        
def fminconset(p_current, bestPoint, p):
    p2 = milpTransfer(p)
    p2.lb, p2.ub = p_current.lb, p_current.ub
    
    
    try:
        r = p2.solve(p.nlpSolver)
        curr_NLP_Point = p.point(r.xf)
        curr_NLP_Point._f = r.ff
    except OpenOptException:
        r = None
    
    resultPoint = p.point(bestPoint.x)
    if r is None or r.istop <0 or curr_NLP_Point.f() >= bestPoint.f(): 
        resultPoint._f  = inf
        return resultPoint
    elif r.istop == 0:
        pass# TODO: fix it
    
    # check if all discrete constraints are satisfied
    x = curr_NLP_Point.x
    k = -1
    for i in p.discreteVars.keys():#range(m):	# check x-vector
        # TODO: replace it by "for i, val in dict.itervalues()"
        if not any(abs(x[i] - p.discreteVars[i]) < p.discrtol):
            k=i	# Violation of this set constraint.
            break # Go and split for this x-component

    if k != -1:		# some discrete constraint violated => recursive search is required
        p.debugmsg('k='+str(k)+' x[k]=' + str(x[k]) + ' p.discreteVars[k]=' +str(p.discreteVars[k]))
        Above=where(p.discreteVars[k]>x[k])[0]
        Below=where(p.discreteVars[k]<x[k])[0]
        resultPoint._f = inf
    else:
        if curr_NLP_Point.f() < bestPoint.f():
            bestPoint = curr_NLP_Point
        p.iterfcn(curr_NLP_Point)
        if p.istop:
            if bestPoint.betterThan(curr_NLP_Point):
                p.iterfcn(bestPoint)
            raise isSolved
        return curr_NLP_Point#bestPoint#x, h, exitflag
    

    # Solve first subproblem if it exists
    if Below.size != 0:
        below = Below[-1]# index of largest set element below x(k)
        
        p3 = milpTransfer(p)
        p3.x0 = x.copy()
        
        ub1 = p_current.ub.copy()
        ub1[k] = min((ub1[k], p.discreteVars[k][below]))	# new upper bound on x[k] for 1st subproblem
        ub1[k] = p.discreteVars[k][below]
        p3.ub = ub1
        p3.lb = p_current.lb
        
        Point_B =  fminconset(p3, bestPoint, p)
        resultPoint = Point_B
        if p.discreteConstraintsAreSatisfied(Point_B.x) and Point_B.betterThan(bestPoint):
            bestPoint = Point_B
            #p.iterfcn(bestPoint)

    # Solve second subproblem if it exists
    if Above.size != 0:
        above = Above[0]# index of smallest set element above x(k)
        
        p4 = milpTransfer(p)
        p4.x0 = x.copy()
        
        lb1=p_current.lb.copy()
        lb1[k]=max((lb1[k], p.discreteVars[k][above]))# new upper bound on x[k] for 1st subproblem
        lb1[k] =  p.discreteVars[k][above]
        p4.lb = lb1
        p4.ub = p_current.ub
        
        Point_A =  fminconset(p4, bestPoint, p)
        resultPoint = Point_A
        if p.discreteConstraintsAreSatisfied(Point_A.x) and Point_A.betterThan(bestPoint):
            bestPoint = Point_A        
            #p.iterfcn(bestPoint)
    
    if Below.size != 0 and Above.size != 0:
        if Point_A.f() < Point_B.f():
            resultPoint = Point_A
        else:
            resultPoint = Point_B
            
    return resultPoint#bestPoint

def milpTransfer(originProb):
    newProb = NLP(originProb.f, originProb.x0)
    originProb.fill(newProb)
    newProb.discreteVars = originProb.discreteVars
    def err(s): # to prevent text output
        raise OpenOptException(s)
    newProb.err = err
    for fn in ['df', 'd2f', 'c', 'dc', 'h', 'dh']:
        if hasattr(originProb, fn) and getattr(originProb.userProvided, fn) or originProb.isFDmodel:
            setattr(newProb, fn, getattr(originProb, fn))
    
    newProb.plot = 0
    newProb.iprint = -1
    newProb.nlpSolver = originProb.nlpSolver 
    return newProb


        
        
        



'''
Copyright (c) 2010 Enzo Michelangeli and IT Vision Ltd

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
'''

from numpy import *
from LCPSolve import LCPSolve

def QPSolve(Q, e, A=None, b=None, Aeq=None, beq=None, lb=None, ub=None):
    '''
    Note: if lb == None lower bounds are assumed to be -Inf
          if ub == None upper bounds are assumed to be +Inf
    i.e., x is NOT assumed to be non-negative by default
    This quadratic solver works by converting the QP problem 
    into an LCP problem. It does well up to few hundred variables
    and dense problems (it doesn't take advantage of sparsity).
    It fails if Aeq is not full row rank, or if Q is singular.
    '''
    nvars = Q.shape[0] # also e.shape[0]
    # convert lb and ub (if present) into Ax <=> b conditions, but
    # skip redundant rows: the ones where lb[i] == -Inf or ub[i] == Inf
    if lb != None:
        delmask = (lb != -Inf)
        addA = compress(delmask, eye(nvars), axis=0)
        addb = compress(delmask, lb, axis=0)
        A = vstack([A, -addA]) if A != None else -addA
        b = concatenate([b, -addb]) if b != None else -addb
    if ub != None:
        delmask = (ub != Inf)
        addA = compress(delmask, eye(nvars), axis=0)
        addb = compress(delmask, ub, axis=0)
        A = vstack([A, addA]) if A != None else addA
        b = concatenate([b, addb]) if b != None else addb

    n_ineq = A.shape[0]
    #print "nett ineq cons:", n_ineq

    # if there are equality constraints, it's equiv to particular MLCP that 
    # can anyway be converted to LCP 
    '''
    minimize:
        e' x + 1/2 x' Q x 
    s.t.:
        A x <= b 
        Aeq x = beq
        
    Upper and lower bounds, if presents, are automatically added to (A, b)

    The Karush-Kuhn-Tucker first-order conditions (being mu and lambda the
    KKT multipliers) are:

        (1.1) e + Q x + Aeq' mu + A' lambda = 0
        (1.2) Aeq x = beq
        (1.3) s = b - A x
        (1.4) s >= 0
        (1.5) lambda >= 0   
        (1.6) s' lambda = 0

    lambda are multipliers of inequality constr., mu of equality constr.,
    s are the slack variables for inequalities.

    This is a MLCP, where s and lambda are complementary. However, 
    we can re-write (1.1) and (1.2) as:

        | Q  Aeq'| * | x |  =  - | e + A' lambda |
        |-Aeq  0 |   |mu |       |       beq       |

    ...and, as long as 

        | Q  Aeq'|
        |-Aeq  0 |

    ...is non-singular (TODO: this should be checked!), we can solve:

        | x | = inv(| Q  Aeq'|) * | -e - A' lambda |
        |mu |       |-Aeq  0 |    |       -beq       |

    Then, if we define:

        M =     | A 0 | * inv( | Q  Aeq'| ) * | A'|
                               |-Aeq  0 |     | 0 |    

        q = b + | A 0 | * inv( | Q  Aeq'| ) * | e |
                               |-Aeq  0 |     | beq |

    ...(1.3) can be rewritten as an LCP problem in (s, lambda):

        s = M lambda + q

    (proof: replace M and q in the eq. above, and simplify remembering
    that | A'| lmbd == | A' lmbd |  , finally reobtaining  s = b - Ax )
         | 0 |         |    0    |

    Now, as we saw,    

        | Q  Aeq'| * | x |  = - | e + A' lambda | 
        |-Aeq  0 |   |mu |      |      beq      |

    ...so we can find

        | x | = inv( | Q  Aeq'| ) * - | e + A' lambda |
        |mu |        |-Aeq    |       |      beq      |

    ...and x will be in the first nvar elements of the solution.
    (we can also verify that b - A x == s)

    The advantage of having an LCP in lambda and s alone is also greater efficiency
    (no mu's and x's are calculated by the Lemke solver). Also, x's are not
    necessarily positive (unlike s's and lambda's), unless there are eplicit A,b
    conditions about it. However, the matrix inversion does cause a loss of accuracy
    (which could be estimated through B's condition number: should this value be 
    returned with the status?).
    
    '''
    if Aeq != None:
        n_eq = Aeq.shape[0]
        B = vstack([
                hstack([ Q, Aeq.T]),
                hstack([-Aeq, zeros((n_eq, n_eq))])
             ])
        A0 = hstack([A, zeros((n_ineq, n_eq))])
    else:
        B = Q
        A0 = A

    #print "det(M):", linalg.det(B)
    #print "B's log10(condition number):", log10(linalg.cond(B))
    BI = linalg.inv(B)

    A0BI = dot(A0, BI)

    M = dot(A0BI, A0.T)
    q = b + dot(A0BI, concatenate((e, beq))) if Aeq != None else b + dot(A0BI, e)

    # LCP: s = M lambda + q, s >= 0, lambda >= 0, s'lambda = 0
    # print "M is",M.shape,", q is ",q.shape
    s, lmbd, retcode = LCPSolve(M,q) 

    if retcode[0] == 1:
        xmu = dot(BI, -concatenate([e + dot(A.T, lmbd), beq])) if Aeq != None else dot(BI, -(e + dot(A.T, lmbd)))
        x = xmu[:nvars]
    else:
        x = None

    return (x, retcode)
    



from baseProblem import MatrixProblem
import numpy as np
from MOP import MOPsolutions

class KSP(MatrixProblem):
    _optionalData = []
    probType = 'KSP'
    expectedArgs = ['objective', 'items']
    allowedGoals = ['min', 'max', 'minimum', 'maximum']
    showGoal = True
    
    def __init__(self, *args, **kw):
        self.goal = 'max'
        self.objective = 'weight'
        MatrixProblem.__init__(self, *args, **kw)
        self.__init_kwargs = kw
        self._immutable = True
        
    def manage(self, *args, **kw):
        kw['routine'] = 'manage'
        r = self.solve(*args, **kw)
        return r
        
    def solve(self, *args, **kw):
        import FuncDesigner as fd, openopt as oo 
        
        if len(args) > 1:
            self.err('''
            incorrect number of arguments for solve(), 
            must be at least 1 (solver), other must be keyword arguments''')
        solver = args[0] if len(args) != 0 else kw.get('solver', self.solver)
        routine = kw.pop('routine', 'solve')

        KW = self.__init_kwargs.copy()
        KW.update(kw)
        items = self.items
        n = len(items)
        objective = KW.get('objective', self.objective)
        if isinstance(objective, (list, tuple, set)):
            nCriteria = len(self.objective)
            if 3 * nCriteria != np.asarray(self.objective).size:
                objective = [(objective[3*i], objective[3*i+1], objective[3*i+2]) for i in range(int(round(np.asarray(self.objective).size / 3)))]
            if len(objective) == 1:
                KW['fTol'], KW['goal'] = objective[0][1:]
        else:
            objective = [(self.objective, KW.get('fTol', getattr(self, 'fTol')), KW.get('goal', getattr(self, 'goal')))]

        nCriteria = len(objective)
        isMOP = nCriteria > 1
        #mainCr = objective[0][0]
        
        solverName = solver if type(solver) == str else solver.__name__
        is_interalg = solverName == 'interalg'
        is_glp = False#solverName == 'sa'
        if is_glp:
            assert nCriteria == 1, 'you cannot solve multiobjective KSP by the solver'
            
        #is_interalg_raw_mode = is_interalg and KW.get('dataHandling', oo.oosolver(solver).dataHandling) in ('auto','raw')
        KW.pop('objective', None)
        P = oo.MOP if nCriteria > 1 else oo.GLP if is_interalg else oo.MILP if not is_glp else oo.GLP

        
        x = fd.oovars(n, domain=bool)
        requireCount  = False
        for i, obj in enumerate(items):
            if 'n' in obj:
                x[i].domain = np.arange(obj['n']+1) if is_interalg else int
                x[i].ub = obj['n']
                x[i].lb = 0
                requireCount  = True

        #cr_values = dict([(obj[0], []) for obj in objective])
        
        constraints = []
        
        Funcs = {}
        Cons = KW.pop('constraints', [])
        if type(Cons) not in (list, tuple):
            Cons = [Cons]
        usedValues = getUsedValues(Cons)
        usedValues.update(getUsedValues([obj[0] for obj in objective]))
        cr_values = {}
        for val in usedValues:
            cr_values[val] = 1 if val == 'nItems' else [obj[val] for obj in items]
        
#        MainCr = mainCr if type(mainCr) in (str, np.str_) else list(usedValues)[0]
#        
#        isMainCrMin = objective[0][2] in ('min', 'minimum')
        
        # handling objective(s)
        FF = []
        
        for optCrName in usedValues:
            F = fd.sum(x * cr_values[optCrName])
            Funcs[optCrName] = F
        
        for obj in objective:
            FF.append((Funcs[obj[0]](obj[0]) if type(obj[0]) in (str, np.str_) else obj[0](Funcs), obj[1], obj[2]))

        for c in Cons:
            tmp = c(Funcs)
            if type(tmp) in (list, tuple, set):
                constraints += list(tmp)
            else:
                constraints.append(tmp)
        
        startPoint = {x:[0]*n}

        p = P(FF if isMOP else FF[0][0], startPoint, constraints = constraints)#, fixedVars = fixedVars)
        if not isMOP:
            p.goal=self.goal
        p.finish = lambda r, p: encode(r, isMOP, n, x, requireCount, items, FF, usedValues)
        r = getattr(p, routine)(solver, **KW)
        if r is None:
            return
        
#        if P != oo.MOP:
#            r.ff = p.ff
        
        return r

class D:
    def __init__(self):
        self.used_vals = set()
    def __getitem__(self, item):
        self.used_vals.add(item)
        return 1.0
    
def getUsedValues(Iterator):
    d = D()
    r = set()
    for elem in Iterator:
        if type(elem) in [str, np.str_]:
            r.add(elem)
        else:
            elem(d)
    r.update(d.used_vals)
    return r
     
def encode(r, isMOP, n, x, requireCount, items, FF, usedValues):
    if isMOP:
        #assert not requireCount, 'MOP with nItem > 1 is unimplemented yet'
        r.solution = 'for MOP see r.solutions instead of r.solution'
        #tmp_c, tmp_v = r.solutions.coords, r.solutions.values  
        if len(r.solutions):
            S = []
            for s in r.solutions:
                tmp = [((i, int(s[x[i]])) if requireCount else i) for i in range(n) if s[x[i]]>=1]
                if 'name' in items[0]:
                    tmp = [(items[i]['name'], k) for i, k in tmp] if requireCount else [items[i]['name'] for i in tmp]
                S.append(dict(tmp) if requireCount else tmp)
            Vals = dict([(ff[0].name, r.solutions.values[:, i]) for i, ff in enumerate(FF)])

            Dicts = [s.copy() for s in S]
            for v in usedValues:
                if v != 'nItems':
                    for i, d in enumerate(Dicts):
                        d[v] = Vals[v][i]
            r.solutions = MOPsolutions(Dicts)
            r.solutions.coords = S
            r.solutions.values = Vals
    else:
        tmp = [((i, int(r.xf[x[i]])) if requireCount else i) for i in range(n) if r.xf[x[i]]>=1]
        if 'name' in items[0]:
            tmp = [(items[i]['name'], k) for i, k in tmp] if requireCount else [items[i]['name'] for i in tmp]
        r.xf = dict(tmp) if requireCount else tmp
    r.probType = 'KSP'



'''
Simple OpenOpt graph stability number example;
requires networkx (http://networkx.lanl.gov)
and FuncDesigner installed.
For maximum clique problem you could use STAB on complementary graph, for networkx it's nx.complement(G) 
Unlike networkx maximum_independent_set() we search for *exact* solution.
Limitations on time, cputime, "enough" value, basic GUI features are available.
'''
from openopt import STAB

import networkx as nx
G = nx.path_graph(15) # [(0,1), (1,2), (2,3), ..., (13,14)]
# you can use string and other types, e.g. 
# G = nx.Graph(); G.add_node('asdf'); G.add_nodes_from(['a1', 'b2','c3']); G.add_edge('qwerty1','qwerty2')
p = STAB(G, includedNodes = [1, 5, 8], excludedNodes = [0, 4, 10])
# or 
# p = STAB(G, includedNodes = ['a1', 'c3'], excludedNodes = ['qwerty1'])

# includedNodes and excludedNodes are optional arguments - nodes that must be present or absent in the solution

# solve by BSD- licensed global nonlinear solver interalg (http://openopt.org/interalg):
r = p.solve('interalg', iprint = 0) # set it to -1 to completely suppress interalg output; may not work for glpk, lpSolve
# or solve by MILP solver:
#r = p.solve('lpSolve') # see http://openopt.org/MILP for other MILP solvers available in OpenOpt

# if your solver is cplex or interalg, you can provide some stop criterion, 
# e.g. maxTime, maxCPUTime, fEnough etc, for example 
# r = p.solve('cplex', maxTime = 100, fEnough = 10) 
# i.e. stop if solution with 10 nodes has been obtained or 100 sec were elapsed
# (gplk and lpSolve has no iterfcn connected and thus cannot handle those criterions)

# also you can use p.manage() to enable basic GUI (http://openopt.org/OOFrameworkDoc#Solving) 
# it requires tkinter installed, that is included into PythonXY, EPD;
# for linux use [sudo] easy_install tk or [sodo] apt-get install python-tk
#r = p.manage('cplex')

print(r.ff, r.solution)
'''
Solver:   Time Elapsed = 0.34   CPU Time Elapsed = 0.28
(6, [1, 3, 5, 8, 12, 14])
for the example with text nodes it will be
['qwerty1', 'a1', 'b2', 'c3', 'asdf']
'''



from numpy import isfinite, any, hstack
from openopt.kernel.baseSolver import *
from sqlcp import sqlcp as SQLCP
from numpy.linalg import LinAlgError
from LCPSolve import LCPSolve

class lcpsolve(baseSolver):
    __name__ = 'lcp'
    __license__ = "MIT"
    __authors__ = "Rob Dittmar, Enzo Michelangeli and IT Vision Ltd"
    __alg__ = "Lemke's Complementary Pivot algorithm"
    __optionalDataThatCanBeHandled__ = []
    #iterfcnConnected = True
    #_canHandleScipySparse = True
    __info__ = '''  '''
    pivtol = 1e-8

    def __init__(self): pass
    def __solver__(self, p):
        w, z, retcode = LCPSolve(p.M,p.q, pivtol=self.pivtol)
        p.xf = hstack((w, z))
        if retcode[0] == 1:
            p.istop = 1000
            p.msg = 'success'
        elif retcode[0] == 2:
            p.istop = -1000
            p.msg = 'ray termination'



"""
pname, e, Q, A=None, b=None, Aeq=None, beq=None, lb=None, ub=None, c0 = MPSparse(filename)

Reads the description of a QP problem from a file in the extended
MPS format (QPS) described by Istvan Maros and Csaba Meszaros at:
http://www.sztaki.hu/~meszaros/public_ftp/qpdata/qpdata.ps

Returns a tuple (pname, e, Q, Aeq, beq, lb, ub, c0)
name: string
all others: numpy arrays

QPS Format:

The QP problems are assumed to be in the following form:

min f(x) = e'x + 1/2 x' Q x,  Q symmetric and positive semidefinite
                                       
subject to      Aeq x = beq,
                l <= x <= u.


After the BOUNDS section of MPS there is an new section introduced
by a QUADOBJ record followed by columnwise elements of Q; row and
columns are column names of A. Being the matrix symmetrical,
only lower triangular elements are listed.

---------------------------------------------------------------------
Field:    1           2          3         4         5         6
Columns:  2-3        5-12      15-22     25-36     40-47     50-61
0        1         2         3         4         5         6
1234567890123456789012345678901234567890123456789012345678901  << column
 11 22222222  33333333  444444444444   55555555  666666666666  << field
---------------------------------------------------------------

          NAME   problem name

          ROWS

           type     name

          COLUMNS
                   column       row       value     row      value
                    name        name                name
          RHS
                    rhs         row       value     row      value
                    name        name                name
          RANGES
                    range       row       value     row      value
                    name        name                name
          BOUNDS

           type     bound       column     value
                    name        name
          ENDATA
---------------------------------------------------------------------


---------------------------------------------------------------
NAME          QP example
ROWS
 N  obj
 G  r1
 L  r2
COLUMNS
 11 22222222  33333333  444444444444   55555555  666666666666  << field
    c1        r1                 2.0   r2                -1.0
    c1        obj                1.5
    c2        r1                 1.0   r2                 2.0
    c2        obj               -2.0
RHS
    rhs1      r1                 2.0   r2                 6.0
BOUNDS
 UP bnd1      c1                20.0
QUADOBJ
    c1        c1                 8.0
    c1        c2                 2.0
    c2        c2                10.0
ENDATA
---------------------------------------------------------------



"""
from numpy import *

def QPSparse(filename):
    pname = e = Q = A = b = Aeq = beq = lb = ub = None  # defaults for ret. params
    c0 = 0.
    f = open(filename, 'r')
    section = None
    rowtype = {}    # dict of row type corresponding to a given row name 
    colnum = {}     # dict of col number corresponding to a given col name
    colcnt = 0      # counter of columns
    acnt = 0        # counter of inequalities (up to size of A matrix)
    aeqcnt = 0      # counter of equalities (up to size of Aeq matrix)
    aindx = {}      # dict of row number in A matrix corresponding to a given row name
    aeqindx = {}    # dict of row number in Aeq matrix corresponding to a given row name
    objcoef = {}    # dict of coefficient in obj function corresponding to a given column name
    RHSdic = {}     # dict containing {row_name, RHSvalue}
    RANGESdic = {}   # dict containing {row_name, RANGEvalue}
    ucnt = 0
    qcnt = 0
    lineno = 0
    for line in f:
        lineno += 1
        line = line.upper().strip("\n\r")
        f1 = line[1:3].strip()
        f2 = line[4:12].strip()
        f3 = line[14:22].strip()
        f4 = line[24:36].strip().replace('D','E')
        f4 = 0. if f4 == "" else float(f4)
        f5 = line[39:47].strip()
        f6 = line[49:61].strip().replace('D','E')
        f6 = 0. if f6 == "" else float(f6)
        
        if line[0] != ' ': # indicator record, switches current section
        
            # see which section is being closed, and take appropriate action
            if section == 'ROWS':  # being closed
                # we know how many rows we have. Allocate lists of lists for A, Aeq
                # Alist[n][colname] contains coeff for row n and col name colname in Aeq
                Alist = [{} for i in range(acnt)] 
                # Aeqlist[n][colname] contains coeff for row n and col name colname in Aeq
                Aeqlist = [{} for i in range(aeqcnt)]
            elif section == 'COLUMNS':  # being closed
                # we know how any columns we have (colcnt). So we can now build Q, ub and lb
                Q = zeros((colcnt,colcnt))
                ub = array([Inf]*colcnt)
                lb = zeros(colcnt)
            elif section == 'RHS':      # being closed
                pass    # b, beq and c0 have been already set up
            elif section == 'RANGES':   # being closed
                pass    # TODO: add ranges
            elif section == 'BOUNDS':   # being closed
                if ucnt == 0:
                    ub = None
            elif section == 'QUADOBJ':  # being closed
                if qcnt == 0:
                    Q = None
                    
            # set the section indicator according to the new section    
            if f1 == 'AM':
                section = 'NAME'    # being opened
            elif f1 == 'OW':
                section = 'ROWS'    # being opened
            elif f1 == 'OL':
                section = 'COLUMNS' # being opened
            elif f1 == 'HS': 
                section = 'RHS'     # being opened
            elif f1 == 'AN': 
                section = 'RANGES'  # being opened
            elif f1 == 'OU': 
                section = 'BOUNDS'  # being opened
            elif f1 == 'UA':
                section = 'QUADOBJ' # being opened
            elif f1 == 'ND':
                section = None
                break;
            else:
                f.close()
                raise(ValueError('invalid indicator record in line '+str(lineno)+': "'+line+'"'))
        elif section == 'NAME':
            pname = f3
        elif section == 'ROWS':
            rname = f2
            if f1 == 'N':
                rowtype[rname] = 'N'
                obj = rname
            elif f1 == 'G':
                rowtype[rname] = 'G'
                aindx[rname] = acnt
                acnt += 1
            elif f1 == 'L':
                rowtype[rname] = 'L'
                aindx[rname] = acnt
                acnt += 1
            elif f1 == 'E':
                rowtype[rname] = 'E'
                aeqindx[rname] = aeqcnt
                aeqcnt += 1
            else:
                f.close()
                raise(ValueError('invalid row type "'+f1+'" in line '+str(lineno)+': "'+line+'"'))
        elif section == 'COLUMNS':
            cname = f2
            rnames = [0,0]
            vals = [0,0]
            if cname not in colnum:
                colnum[cname] = colcnt  # alocate a new column number
                colcnt += 1
            rnames[0] = f3
            vals[0] = f4
            rnames[1] = f5
            vals[1] = f6
            for i in (0,1): # both are possible
                rn = rnames[i]
                value = vals[i]
                if rn == '':
                    break
                if rn == obj: # then value is the coefficient of col cname in the obj function
                    objcoef[cname] = value
                elif rowtype[rn] == 'L': #
                    Alist[aindx[rn]][cname] = value
                elif rowtype[rn] == 'G': #
                    Alist[aindx[rn]][cname] = -value
                elif rowtype[rn] == 'E': #
                    Aeqlist[aeqindx[rn]][cname] = value
        elif section == 'RHS':
            # What is the RHS name for????
            rnames = [0,0]
            vals = [0,0]
            rnames[0] = f3
            vals[0] = f4
            rnames[1] = f5
            vals[1] = f6
            for i in (0,1): # both are possible
                rname = rnames[i]
                value = vals[i]
                if rname == '':
                    break
                RHSdic[rname] = value
        elif section == 'RANGES':
            # What is the RANGE name for????
            rnames = [0,0]
            vals = [0,0]
            rnames[0] = f3
            vals[0] = f4
            rnames[1] = f5
            vals[1] = f6
            for i in (0,1): # both are possible
                rname = rnames[i]
                value = vals[i]
                if rname == '':
                    break
                RANGESdic[rname] = value
        elif section == 'BOUNDS':
            # by default, x >= 0
            # what is the Bound name in f2 for??? 
            # UP : x <= b, x >= 0
            # LO :         x >= b
            # FX : x == b
            # FR : (no bound: remove default >= 0)
            # MI : x > -Inf
            # BV : x in (0, 1) # NOT SUPPORTED
            ic = colnum[f3]
            val = f4
            if f1 == 'UP':
                ub[ic] = f4
                ucnt += 1
            elif f1 == 'LO':
                lb[ic] = f4
            elif f1 == 'FX':
                # TODO add an equality constraint
                raise(ValueError('fixed variable (FX) bound not supported in line '+str(lineno)+': "'+line+'"'))
            elif f1 == 'FR':
                lb[ic] = -Inf
                ub[ic] = Inf
            elif f1 == 'MI':
                lb[ic] = -Inf
            elif f1 == 'BV':
                raise(ValueError('binary value (BV) bound not supported in line '+str(lineno)+': "'+line+'"'))
                
        elif section == 'QUADOBJ':
            ic1 = colnum[f2]
            ic2 = colnum[f3]
            val = f4
            Q[ic1,ic2] = val
            if ic1 != ic2:    # if not on diagonal
                Q[ic2,ic1] = val
            qcnt += 1
    f.close()
    if section != None:
        raise(EOFError('unexpected EOF while in section '+section))
        
   # Now we have all necessary info and we can build A,b,Aeq and Beq      
    if acnt > 0:
        A = zeros((acnt, colcnt))
        b = zeros(acnt)
        for rn in range(acnt):
            for c in Alist[rn]:
                A[rn, colnum[c]] = Alist[rn][c]
    if aeqcnt > 0:
        Aeq = zeros((aeqcnt, colcnt))
        beq = zeros(aeqcnt)
        for rn in range(aeqcnt):
            for c in Aeqlist[rn]:
                Aeq[rn, colnum[c]] = Aeqlist[rn][c]
        
    # ########## process RHS
    for rname in RHSdic:
        value = RHSdic[rname]
        rt = rowtype[rname]
        if rt == 'L': #
            b[aindx[rname]] = value  # constant term in obj function
        elif rt == 'G': #
            b[aindx[rname]] = -value
        elif rt == 'E': #
            beq[aeqindx[rname]] = value
        elif rt == 'N':
            c0 = -value # constant term in obj function
        
    # Handle RANGE lines. Each range causes a duplicate of a 
    # row in A and b, and a modification of original and new b.
    """
    D. The RANGES section is for constraints of the form:  h <= constraint <= u .
       The range of the constraint is  r = u - h .  The value of r is specified
       in the RANGES section, and the value of u or h is specified in the RHS
       section.  If b is the value entered in the RHS section, and r is the
       value entered in the RANGES section, then u and h are thus defined:

            row type       sign of r       h          u
            ----------------------------------------------
               L            + or -       b - |r|      b
               G            + or -         b        b + |r|
               E              +            b        b + |r|
               E              -          b - |r|      b
    """
    if A != None:
        addA = zeros((len(RANGESdic),A.shape[1]))
        addb = zeros(len(RANGESdic))

        for rname in RANGESdic:
            index = aindx[rname]    # row index in A and index in b 
            value = RANGESdic[rname]
            rt = rowtype[rname]
            if rt == 'L': #
                b1 = b[index] + abs(value) # sign???
            elif rt == 'G': #
                b1 = b[index] - abs(value) # sign???
            elif rt == 'E': #
                raise(ValueError('RANGES for rows of type E not yet supported in line '+str(lineno)+': "'+line+'"'))
                #b1 = b[index] - value

            addA[index,:] = -A[index,:]    # append to A duplicate of index row, with sign changed
            addb[index]   = -b1            # append to b other extreme, with sign changed

        A = vstack([A, addA])
        b = concatenate([b, addb])     
    
    e = zeros(colcnt)
    for c in objcoef:
        e[colnum[c]] = objcoef[c]

    # suppress redundant lb/ub
    nvars = e.shape[0]
    if lb != None and (lb == array([-Inf]*nvars)).all():
        lb = None
    if ub != None and (ub == array([Inf]*nvars)).all():
        ub = None
        
    return (pname, e, Q, A, b, Aeq, beq, lb, ub, c0)





from oct2py import Oct2Py
oc = Oct2Py()
from openopt.kernel.baseSolver import baseSolver
from openopt.kernel.nonOptMisc import isspmatrix
import numpy as np

class octave_qp(baseSolver):
    __name__ = 'octave_qp'
    __alg__ = ''
    __authors__ = ''
#    __isIterPointAlwaysFeasible__ = True
    
    __optionalDataThatCanBeHandled__ = ['lb', 'ub', 'A', 'Aeq', 'b', 'beq']
#    properTextOutput = True
    #_canHandleScipySparse = True

    def __init__(self): pass
    def __solver__(self, p):
        x, obj, info, Lambda = oc.qp(
                                     p.x0.reshape(-1, 1), 
                                     p.H.tocsr() if isspmatrix(p.H) else np.asfarray(p.H), 
                                     np.asfarray(p.f).reshape(-1, 1), 
                                     p.Aeq.tocsr() if isspmatrix(p.Aeq) else np.asfarray(p.Aeq) if p.nbeq != 0 else [], 
                                     p.beq.reshape(-1, 1) if p.nbeq != 0 else [], 
                                     p.lb.reshape(-1, 1), 
                                     p.ub.reshape(-1, 1), 
                                     [], 
                                     p.A.tocsr() if isspmatrix(p.A) else np.asfarray(p.A) if p.nb != 0 else [1.0]*p.n, 
                                     p.b.reshape(-1, 1) if p.nb != 0 else np.inf#p.b.reshape(p.n, 0)
                                     )
        p.xk = p.xf = x.flatten()
        p.istop = 1000



"""
OpenOpt SOCP example
for the problem http://openopt.org/images/2/28/SOCP.png
"""

from numpy import *
from openopt import SOCP

f = array([-2, 1, 5])

C0 = mat('-13 3 5; -12 12 -6')
d0 = [-3, -2]
q0 = array([-12, -6, 5])
s0 = -12

C1 = mat('-3 6 2; 1 9 2; -1 -19 3')
d1 = [0, 3, -42]
q1 = array([-3, 6, -10])
s1 = 27

p = SOCP(f,  C=[C0, C1],  d=[d0, d1], q=[q0, q1], s=[s0, s1]) 
# you could add lb <= x <= ub, Ax <= b, Aeq x = beq constraints 
# via p = SOCP(f,  ..., A=A, b=b, Aeq=Aeq, beq=beq,lb=lb, ub=ub)
r = p.solve('cvxopt_socp')
x_opt, f_opt = r.xf,  r.ff
print(' f_opt: %f    x_opt: %s' % (f_opt, x_opt))
# f_opt: -38.346368    x_opt: [-5.01428121 -5.76680444 -8.52162517]



from baseProblem import MatrixProblem

class MCP(MatrixProblem):
    _optionalData = []
    probType = 'MCP'
    expectedArgs = ['graph']
    allowedGoals = ['maximum clique']
    showGoal = False

    _init = False
    
    def __setattr__(self, attr, val): 
        if self._init: self.err('openopt MCP instances are immutable, arguments should pass to constructor or solve()')
        self.__dict__[attr] = val

    def __init__(self, *args, **kw):
        MatrixProblem.__init__(self, *args, **kw)
        self.__init_kwargs = kw
        self._init = True
        
    def manage(self, *args, **kw):
        kw['routine'] = 'manage'
        r = self.solve(*args, **kw)
        return r
        
    def solve(self, *args, **kw):
        import networkx as nx
        graph = nx.complement(self.graph)
        from openopt import STAB
        KW = self.__init_kwargs
        KW.update(kw)
        P = STAB(graph, **KW)
        r = P.solve(*args)
        return r
        



"""
Solve the least-squares problem

  minimize ||Ax-b||

using LSQR.  This is a line-by-line translation from Matlab code
available at http://www.stanford.edu/~saunders/lsqr.

Michael P. Friedlander, University of British Columbia
Dominique Orban, Ecole Polytechnique de Montreal
$Id$
"""

from numpy import zeros, dot
from openopt.kernel.ooMisc import norm
from math import sqrt

# Simple shortcuts---linalg.norm is too slow for small vectors
def normof2(x,y): return sqrt(x**2 + y**2)
def normof4(x1,x2,x3,x4): return sqrt(x1**2 + x2**2 + x3**2 + x4**2)

def lsqr( m, n, aprod, b, damp, atol, btol, conlim, itnlim, show, wantvar = False, callback = lambda x: None):
    """
    [ x, istop, itn, r1norm, r2norm, anorm, acond, arnorm, xnorm, var ]...
    = lsqr( m, n, @aprod, b, damp, atol, btol, conlim, itnlim, show );
    
    LSQR solves  Ax = b  or  min ||b - Ax||_2  if damp = 0, or

       min || [ b ]  -  [   A    ] x ||   otherwise.
           || [ 0 ]     [ damp I ]   ||2

    A  is an m by n matrix defined by  y = aprod(mode, m, n, x),
    where aprod refers to a function that performs the matrix-vector operations.
    If mode = 1,   aprod  must return  y = Ax   without altering x.
    If mode = 2,   aprod  must return  y = A'x  without altering x.

    ----------------------------------------------------------------------
    LSQR uses an iterative (conjugate-gradient-like) method.

    For further information, see 

    1. C. C. Paige and M. A. Saunders (1982a).
       LSQR: An algorithm for sparse linear equations and sparse least squares,
       ACM TOMS 8(1), 43-71.
    2. C. C. Paige and M. A. Saunders (1982b).
       Algorithm 583.  LSQR: Sparse linear equations and least squares problems,
       ACM TOMS 8(2), 195-209.
    3. M. A. Saunders (1995).  Solution of sparse rectangular systems using
       LSQR and CRAIG, BIT 35, 588-604.

    Input parameters:

    atol, btol  are stopping tolerances.  If both are 1.0e-9 (say),
                the final residual norm should be accurate to about 9 digits.
                (The final x will usually have fewer correct digits,
                depending on cond(A) and the size of damp.)
    conlim      is also a stopping tolerance.  lsqr terminates if an estimate
                of cond(A) exceeds conlim.  For compatible systems Ax = b,
                conlim could be as large as 1.0e+12 (say).  For least-squares
                problems, conlim should be less than 1.0e+8.
                Maximum precision can be obtained by setting
                atol = btol = conlim = zero, but the number of iterations
                may then be excessive.
    itnlim      is an explicit limit on iterations (for safety).
    show        if set to 1, gives an iteration log.
                If set to 0, suppresses output.

    Output parameters:

    x           is the final solution.
    istop       gives the reason for termination.
    istop       = 1 means x is an approximate solution to Ax = b.
                = 2 means x approximately solves the least-squares problem.
                r1norm      = norm(r), where r = b - Ax.
                r2norm      = sqrt( norm(r)^2  +  damp^2 * norm(x)^2 )
                = r1norm if damp = 0.
    anorm       = estimate of Frobenius norm of Abar = [  A   ].
                                                       [damp*I]
    acond       = estimate of cond(Abar).
    arnorm      = estimate of norm(A'*r - damp^2*x).
    xnorm       = norm(x).
    var         (if present) estimates all diagonals of (A'A)^{-1} (if damp=0)
                or more generally (A'A + damp^2*I)^{-1}.
                This is well defined if A has full column rank or damp > 0.
                (Not sure what var means if rank(A) < n and damp = 0.)
                
    ----------------------------------------------------------------------
    """

    # Initialize.

    msg=['The exact solution is  x = 0                              ',
         'Ax - b is small enough, given atol, btol                  ',
         'The least-squares solution is good enough, given atol     ',
         'The estimate of cond(Abar) has exceeded conlim            ',
         'Ax - b is small enough for this machine                   ',
         'The least-squares solution is good enough for this machine',
         'Cond(Abar) seems to be too large for this machine         ',
         'The iteration limit has been reached                      ']

    if wantvar:
        var = zeros(n,1)
    else:
        var = None
    
#    if show:
#        print ' '
#        print 'LSQR            Least-squares solution of  Ax = b'
#        str1 = 'The matrix A has %8g rows  and %8g cols' % (m, n)
#        str2 = 'damp = %20.14e     wantvar = %-5s' % (damp, repr(wantvar))
#        str3 = 'atol = %8.2e                 conlim = %8.2e' % (atol, conlim)
#        str4 = 'btol = %8.2e                 itnlim = %8g' % (btol, itnlim)
#        print str1;   print str2;   print str3;   print str4;
        
    itn    = 0;   istop  = 0;   nstop  = 0
    ctol   = 0.0
    if conlim > 0.0: ctol = 1.0/conlim
    anorm  = 0.;	acond  = 0.
    dampsq = damp**2;	ddnorm = 0.;		res2   = 0.
    xnorm  = 0.;	xxnorm = 0.;		z      = 0.
    cs2    = -1.;	sn2    = 0.
    
    # Set up the first vectors u and v for the bidiagonalization.
    # These satisfy  beta*u = b,  alfa*v = A'u.
    
    u      = b[:m];	x    = zeros(n)
    alfa   = 0.;	beta = norm( u )
    if beta > 0:
        u = (1.0/beta) * u;	v = aprod(2, m, n, u)
        alfa = norm( v );

    if alfa > 0:
        v = (1.0/alfa) * v;    w = v.copy();
        
    arnorm = alfa * beta;
    if arnorm == 0:
#        print(msg[0])
        return (x, istop, itn, r1norm, r2norm, anorm, acond, arnorm, xnorm, var)

    rhobar = alfa;		phibar = beta;		bnorm  = beta;
    rnorm  = beta
    r1norm = rnorm
    r2norm = rnorm
    head1  = '   Itn      x(1)       r1norm     r2norm '
    head2  = ' Compatible   LS      Norm A   Cond A'
    
    if show:
#        print ' '
#        print head1+head2
        test1  = 1.0;		test2  = alfa / beta
        str1   = '%6g %12.5e'     % (itn,   x[0])
        str2   = ' %10.3e %10.3e' % (r1norm, r2norm)
        str3   = '  %8.1e %8.1e'  % (test1,  test2)
#        print str1+str2+str3
        
    # ------------------------------------------------------------------
    #     Main iteration loop.
    # ------------------------------------------------------------------
    while itn < itnlim:
        itn = itn + 1
        #   Perform the next step of the bidiagonalization to obtain the
        #   next  beta, u, alfa, v.  These satisfy the relations
        #              beta*u  =  a*v   -  alfa*u,
        #              alfa*v  =  A'*u  -  beta*v.

        u    = aprod(1, m, n, v)  -  alfa*u
        beta = norm( u );
        if beta > 0:
            u     = (1.0/beta) * u
            anorm = normof4(anorm, alfa, beta, damp)
            v     = aprod(2, m, n, u)  -  beta*v
            alfa  = norm( v )
            if alfa > 0:  v = (1.0/alfa) * v
        
        # Use a plane rotation to eliminate the damping parameter.
        # This alters the diagonal (rhobar) of the lower-bidiagonal matrix.

        rhobar1 = normof2(rhobar, damp)
        cs1     = rhobar / rhobar1
        sn1     = damp   / rhobar1
        psi     = sn1 * phibar
        phibar  = cs1 * phibar
        
        #  Use a plane rotation to eliminate the subdiagonal element (beta)
        # of the lower-bidiagonal matrix, giving an upper-bidiagonal matrix.
        
        rho     =   normof2(rhobar1, beta)
        cs      =   rhobar1/ rho
        sn      =   beta   / rho
        theta   =   sn * alfa
        rhobar  = - cs * alfa
        phi     =   cs * phibar
        phibar  =   sn * phibar
        tau     =   sn * phi
        
        # Update x and w.
            
        t1      =   phi  /rho;
        t2      = - theta/rho;
        dk      =   (1.0/rho)*w;
            
        x       = x      +  t1*w
        w       = v      +  t2*w
        ddnorm  = ddnorm +  norm(dk)**2
        if wantvar: var = var  +  dk*dk
            
        # Use a plane rotation on the right to eliminate the
        # super-diagonal element (theta) of the upper-bidiagonal matrix.
        # Then use the result to estimate  norm(x).
            
        delta   =   sn2 * rho
        gambar  = - cs2 * rho
        rhs     =   phi  -  delta * z
        zbar    =   rhs / gambar
        xnorm   =   sqrt(xxnorm + zbar**2)
        gamma   =   normof2(gambar, theta)
        cs2     =   gambar / gamma
        sn2     =   theta  / gamma
        z       =   rhs    / gamma
        xxnorm  =   xxnorm  +  z**2
            
        # Test for convergence.
        # First, estimate the condition of the matrix  Abar,
        # and the norms of  rbar  and  Abar'rbar.
        
        acond   =   anorm * sqrt( ddnorm )
        res1    =   phibar**2
        res2    =   res2  +  psi**2
        rnorm   =   sqrt( res1 + res2 )
        arnorm  =   alfa * abs( tau )
        
        # 07 Aug 2002:
        # Distinguish between
        #    r1norm = ||b - Ax|| and
        #    r2norm = rnorm in current code
        #           = sqrt(r1norm^2 + damp^2*||x||^2).
        #    Estimate r1norm from
        #    r1norm = sqrt(r2norm^2 - damp^2*||x||^2).
        # Although there is cancellation, it might be accurate enough.
        
        r1sq    =   rnorm**2  -  dampsq * xxnorm
        r1norm  =   sqrt( abs(r1sq) )
        if r1sq < 0: r1norm = - r1norm
        r2norm  =   rnorm
        
        # Now use these norms to estimate certain other quantities,
        # some of which will be small near a solution.
        
        test1   =   rnorm / bnorm
        test2   =   arnorm/( anorm * rnorm )
        test3   =       1.0 / acond
        t1      =   test1 / (1    +  anorm * xnorm / bnorm)
        rtol    =   btol  +  atol *  anorm * xnorm / bnorm
        
        # The following tests guard against extremely small values of
        # atol, btol  or  ctol.  (The user may have set any or all of
        # the parameters  atol, btol, conlim  to 0.)
        # The effect is equivalent to the normal tests using
        # atol = eps,  btol = eps,  conlim = 1/eps.
        
        if itn >= itnlim:   istop = 7
        if 1 + test3  <= 1: istop = 6
        if 1 + test2  <= 1: istop = 5
        if 1 + t1     <= 1: istop = 4
        
        # Allow for tolerances set by the user.
        
        if  test3 <= ctol:  istop = 3
        if  test2 <= atol:  istop = 2
        if  test1 <= rtol:  istop = 1
        
        # See if it is time to print something.
            
        prnt = False;
        if n     <= 40       : prnt = True
        if itn   <= 10       : prnt = True
        if itn   >= itnlim-10: prnt = True
        if itn % 10 == 0     : prnt = True
        if test3 <=  2*ctol  : prnt = True
        if test2 <= 10*atol  : prnt = True
        if test1 <= 10*rtol  : prnt = True
        if istop !=  0       : prnt = True
        
        if prnt and show:
            str1 = '%6g %12.5e'     %(   itn,   x[0] )
            str2 = ' %10.3e %10.3e' %(r1norm, r2norm )
            str3 = '  %8.1e %8.1e'  %( test1,  test2 )
            str4 = ' %8.1e %8.1e'   %( anorm,  acond )
#            print str1+str2+str3+str4
                
        if istop > 0: break
        callback(x) # added for OpenOpt kernel

    # End of iteration loop.
    # Print the stopping condition.
        
#    if show:
#        print ' '
#        print 'LSQR finished'
#        print msg[istop]
#        print ' '
#        str1 = 'istop =%8g   r1norm =%8.1e'   %(istop, r1norm )
#        str2 = 'anorm =%8.1e   arnorm =%8.1e' %(anorm, arnorm )
#        str3 = 'itn   =%8g   r2norm =%8.1e'   %(  itn, r2norm )
#        str4 = 'acond =%8.1e   xnorm  =%8.1e' %(acond, xnorm  )
#        str5 = '                  bnorm  =%8.1e'    % bnorm
#        print str1 + '   ' + str2
#        print str3 + '   ' + str4
#        print str5
#        print ' '
        
    return ( x, istop, itn, r1norm, r2norm, anorm, acond, arnorm, xnorm, var )



from baseProblem import NonLinProblem
from ooMisc import assignScript
from numpy import dot, asfarray, atleast_2d, array, zeros
import NLP

class DFP(NonLinProblem):
    _optionalData = ['lb', 'ub', 'A', 'Aeq', 'b', 'beq', 'c', 'h']
    probType = 'DFP'
    expectedArgs = ['f', 'x0', 'X', 'Y']
    allowedGoals = ['minimum', 'min']
    goal = 'minimum'
    showGoal = False
    isObjFunValueASingleNumber = False
    
    def _Prepare(self):
        self.X = atleast_2d(self.X)
        self.Y = array(self.Y, float)
        if self.X.shape[0] != self.Y.shape[0]:
            if self.X.shape[1] != self.Y.shape[0]:
                self.err('incorrect shape of input data')
            else:
                self.X = self.X.T
        NonLinProblem._Prepare(self)
        #if self.Y.ndim

        if self.userProvided.df:
            assert len(self.user.df) == 1
            self.dfff = self.user.df[0]
            def dff(x):
                r = zeros(self.n)
                for i in range(self.Y.shape[0]):
                    #print asfarray(self.fff(x, self.X[i])-self.Y[i]), asfarray(self.dfff(x, self.X[i]))
                    r += dot(2.0 * asfarray(self.fff(x, self.X[i])-self.Y[i]), asfarray(self.dfff(x, self.X[i])))
                return r        
            self.df = self.user.df = dff

    def __finalize__(self):
        NonLinProblem.__finalize__(self)
        if self.userProvided.df:
            self.df = self.dfff
            self.f = self.fff

    def __init__(self, *args, **kwargs):
        NonLinProblem.__init__(self, *args, **kwargs)
        assignScript(self, kwargs)
        self.fff = self.f
        def ff(x):
            r = []
            for i in range(self.Y.shape[0]):
                r.append(asfarray(self.fff(x, self.X[i])-self.Y[i])**2)
            return r
        self.f = ff

    def objFuncMultiple2Single(self, fv):
        assert all(fv.flatten()>=0)
        return fv.sum() # they are squared in other place

    def dfp2nlp(self, solver, **solver_params):
        ff = lambda x: (asfarray(self.f(x))).sum()
        if self.userProvided.df:
            #dff = lambda x: dot(2*asfarray(ff(x)), asfarray(self.df(x)))
            p = NLP.NLP(ff, self.x0, df=self.df)
        else:
            p = NLP.NLP(ff, self.x0)
            
        #p = NLP.NLP(lambda x: self.f(x), self.x0)
        #if self.userProvided.df: p.df = dff
        self.fill(p, sameConstraints=True)
        p.f = ff # to prefent overwriting

        def dfp_iterfcn(*args,  **kwargs):
            self.iterfcn(*args,  **kwargs)
            if self.istop != 0: p.istop, p.msg = self.istop, self.msg
            tmp_iterfcn(*args,  **kwargs)
            if p.istop != 0: self.istop, self.msg = p.istop, p.msg

        p.iterfcn, tmp_iterfcn = dfp_iterfcn, p.iterfcn

        #p.iprint = -1
        self.iprint = -1
        if self.plot: 
            self.plot, p.plot = 0, 1
            p.show = self.show
        #p.show = False
        
        p.checkdf()
        
        r = p.solve(solver, **solver_params)
        #r.ff = ff(r.xf)

        return r



from numpy import hstack,  asarray, abs, atleast_1d, \
logical_not, argsort, vstack, sum, array, nan, all
import numpy as np
from FuncDesigner import oopoint, FDmisc
where = FDmisc.where
#from FuncDesigner.boundsurf import boundsurf


def interalg_ODE_routine(p, solver):
    isIP = p.probType == 'IP'
    isODE = p.probType == 'ODE'
    if isODE:
        f, y0, r30, ftol = p.equations, p.x0, p.times, p.ftol
        assert len(f) == 1, 'multiple ODE equations are unimplemented for FuncDesigner yet'
        f = list(f.values())[0]
        t = list(f._getDep())[0]
    elif isIP:
        assert p.n == 1 and p.__isNoMoreThanBoxBounded__()
        f, y0, ftol = p.user.f[0], 0.0, p.ftol
        if p.fTol is not None: ftol = p.fTol
        t = list(f._getDep())[0]
        r30 = p.domain[t]
        p.iterfcn(p.point([nan]*p.n))
    else:
        p.err('incorrect prob type for interalg ODE routine') 
    
    eq_var = list(p._x0.keys())[0]

    dataType = solver.dataType
    if type(ftol) == int: 
        ftol = float(ftol) # e.g. someone set ftol = 1
    # Currently ftol is scalar, in future it can be array of same length as timeArray
    if len(r30) < 2:
        p.err('length ot time array must be at least 2')    
#    if any(r30[1:] < r30[:-1]):
#        p.err('currently interalg can handle only time arrays sorted is ascending order')  
#    if any(r30 < 0):
#        p.err('currently interalg can handle only time arrays with positive values')  
#    if p.times[0] != 0:
#        p.err('currently solver interalg requires times start from zero')  
    
    r37 = abs(r30[-1] - r30[0])
#    if len(r30) == 2:
#        r30 = np.linspace(r30[0], r30[-1], 150)
    r28 = asarray(atleast_1d(r30[:-1]), dataType)
    r29 = asarray(atleast_1d(r30[1:]), dataType)
    
    r20_store = array([], float)
    r38_store = array([], float)
    r39_store = array([], float)
    maxActiveNodes = 150000#solver.maxActiveNodes

    storedr28 = []
    r27 = []
    r31 = []
    r32 = []
    r33 = ftol
    F = 0.0
    p._Residual = 0
    
    # Main cycle
    for itn in range(p.maxIter+1):
        p.extras['nNodes'].append(r28.size)
        p.extras['nActiveNodes'].append(r28.size)
        mp = oopoint(
                     {t: [r28, r29] if r30[-1] > r30[0] else [r29, r28]}, 
                     skipArrayCast = True
                     )
        mp.isMultiPoint = True
        mp.nPoints = r28.size
        
        mp.dictOfFixedFuncs = p.dictOfFixedFuncs
        mp._dictOfRedirectedFuncs = p._dictOfRedirectedFuncs
        mp.maxDistributionSize = p.maxDistributionSize
        mp.surf_preference = True
        tmp = f.interval(mp, ia_surf_level = 2 if isIP else 1)
        if not all(tmp.definiteRange):
            p.err('''
            solving ODE and IP by interalg is implemented for definite (real) range only, 
            no NaN values in integrand are allowed''')
        # TODO: perform check on NaNs
        isBoundsurf = hasattr(tmp, 'resolve')
        if isBoundsurf:
            if isIP:
                if tmp.level == 1:
                    #adjustr4WithDiscreteVariables(wr4, p)
                    cs = oopoint([(v, asarray(0.5*(val[0] + val[1]), dataType)) for v, val in mp.items()])
                    cs.dictOfFixedFuncs = p.dictOfFixedFuncs
                    cs._dictOfRedirectedFuncs = p._dictOfRedirectedFuncs
                    r21, r22 = tmp.values(cs)
                    o, a = atleast_1d(r21), atleast_1d(r22)
                    r20 = a-o
                    approx_value = 0.5*(a+o)
                else:
                    assert tmp.level == 2
                    ts, te = r28, r29
                    A, B = (te**2 + te*ts+ts**2) / 3.0, 0.5 * (te + ts)
                    a, b, c = tmp.l.d2.get(t, 0.0), tmp.l.d.get(t, 0.0), tmp.l.c
                    val_l = a * A + b * B + c 
                    a, b, c = tmp.u.d2.get(t, 0.0), tmp.u.d.get(t, 0.0), tmp.u.c
                    val_u =  a * A + b * B + c 
                    r20 = val_u - val_l
                    approx_value = 0.5 * (val_l + val_u)
#                    import pylab, numpy
#                    xx = numpy.linspace(-1, 0, 1000)
#                    pylab.plot(xx, tmp.l.d2.get(t, 0.0)[1]*xx**2+ tmp.l.d.get(t, 0.0)[1]*xx+ tmp.l.c[1], 'r')
#                    pylab.plot(xx, tmp.u.d2.get(t, 0.0)[1]*xx**2+ tmp.u.d.get(t, 0.0)[1]*xx+ tmp.u.c[1], 'b')
#                    pylab.grid()
#                    pylab.show()
                    
            elif isODE:
                l, u = tmp.l, tmp.u
                assert len(l.d) <= 1 and len(u.d) <= 1 # at most time variable
                l_koeffs, u_koeffs = l.d.get(t, 0.0), u.d.get(t, 0.0)
                l_c, u_c = l.c, u.c
#                dT = r29 - r28 if r30[-1] > r30[0] else r28 - r29
                
                ends = oopoint([(v, asarray(val[1], dataType)) for v, val in mp.items()])
                ends.dictOfFixedFuncs = p.dictOfFixedFuncs
                ends._dictOfRedirectedFuncs = p._dictOfRedirectedFuncs
                ends_L, ends_U = tmp.values(ends)
                
                starts = oopoint([(v, asarray(val[0], dataType)) for v, val in mp.items()])
                starts.dictOfFixedFuncs = p.dictOfFixedFuncs
                starts._dictOfRedirectedFuncs = p._dictOfRedirectedFuncs
                starts_L, starts_U = tmp.values(starts)

#                o, a = atleast_1d(r21), atleast_1d(r22)

                o, a = tmp.resolve()[0]
#                r20 = 0.5 * u_koeffs * dT  + u_c  - (0.5 * l_koeffs * dT  + l_c)
                r20_end = 0.5 * (ends_U - ends_L)
                r20_start = 0.5 * (starts_U - starts_L)
                r20 = where(r20_end>r20_start, r20_end, r20_start)
                
#                r20 = 0.5 * u_koeffs * dT ** 2 + u_c * dT - (0.5 * l_koeffs * dT ** 2 + l_c * dT)
#                r20 =  0.5*u_koeffs * dT  + u_c  - ( 0.5*l_koeffs * dT  + l_c)

#                o = 0.5*l_koeffs * dT + l_c
#                a = 0.5*u_koeffs * dT + u_c
                #assert 0, 'unimplemented'
            else:
                assert 0
        else:
            o, a = atleast_1d(tmp.lb), atleast_1d(tmp.ub)
            ends_L = starts_L = o
            ends_U = starts_U = a
            r20 = a - o
            approx_value = 0.5 * (a+o)
        
        if isODE:
            r36 = atleast_1d(r20 <= 0.95 * r33)
            r36 = np.logical_and(r36, r20 < ftol)
            r36 = np.logical_and(r36, a-o < ftol)
#        else:
#            r36 = atleast_1d(r20 <= 0.95 * r33 / r37)

        
        if isODE and isBoundsurf:
            d = r37 #if not isODE or not isBoundsurf else len(r28)
            r36 = np.logical_and(
                                atleast_1d(r20_end <= 0.95 * r33 / d), 
                                atleast_1d(r20_start <= 0.95 * r33 / d)
                                )
            r36 &= atleast_1d(r20_end <= ftol)
            r36 &= atleast_1d(r20_start <= ftol)
        else:
            r36 = atleast_1d(r20 <= 0.95 * r33 / r37)
            
#            r36 = np.logical_and(r36, r20 < ftol)
#            r36 = np.logical_and(r36, a-o < ftol)

        ind = where(r36)[0]
        if isODE:
            storedr28.append(r28[ind])
            r27.append(r29[ind])
            r31.append(a[ind])
            r32.append(o[ind])
#            r31.append(ends_U[ind])
#            r32.append(ends_L[ind])
        else:
            assert isIP
            #F += 0.5 * sum((r29[ind]-r28[ind])*(a[ind]+o[ind]))
            F += sum((r29[ind]-r28[ind])*approx_value[ind])
        
        if ind.size != 0: 
            tmp = abs(r29[ind] - r28[ind])
            Tmp = sum(r20[ind] * tmp) #if not isODE or not isBoundsurf else sum(r20[ind])
            r33 -= Tmp
            if isIP: p._residual += Tmp
            r37 -= sum(tmp)
        
        ind = where(logical_not(r36))[0]
        if 1:#new
            if ind.size == 0 and r20_store.size == 0:
                p.istop = 1000
                p.msg = 'problem has been solved according to required user-defined accuracy %0.1g' % ftol
                break
            if ind.size != 0:
                # TODO: use merge sorted lists
                if r20_store.size != 0:
                    r20_store = hstack((r20_store, r20[ind]*abs(r29[ind] - r28[ind])))
                    r38_store =  hstack((r38_store, r28[ind]))
                    r39_store =  hstack((r39_store, r29[ind]))
                else:
                    r20_store = r20[ind]*abs(r29[ind] - r28[ind])
                    r38_store, r39_store = r28[ind], r29[ind]
                ind_a = argsort(r20_store)
                r20_store = r20_store[ind_a]
                r38_store = r38_store[ind_a]
                r39_store = r39_store[ind_a]
            r38_store, r38 = r38_store[:-maxActiveNodes], r38_store[-maxActiveNodes:]
            r39_store, r39 = r39_store[:-maxActiveNodes], r39_store[-maxActiveNodes:]
            r20_store = r20_store[:-maxActiveNodes]
            r40 = 0.5 * (r38 + r39)
            r28 = vstack((r38, r40)).flatten()
            r29 = vstack((r40, r39)).flatten()
        else:
            if ind.size == 0:
                p.istop = 1000
                p.msg = 'problem has been solved according to required user-defined accuracy %0.1g' % ftol
                break
                
            r38, r39 = r28[ind], r29[ind]
            r40 = 0.5 * (r38 + r39)
            r28 = vstack((r38, r40)).flatten()
            r29 = vstack((r40, r39)).flatten()
            
        # !!! unestablished !!!
        if isODE:
            p.iterfcn(fk = r33/ftol)
        elif isIP:
            p.iterfcn(xk=array(nan), fk=F, rk = ftol - r33)
        else:
            p.err('bug in interalgODE.py')
            
        if p.istop != 0 : 
            break
        
        #print(itn, r28.size)

    if isODE:
        
        t0, t1, lb, ub = hstack(storedr28), hstack(r27), hstack(r32), hstack(r31)
        ind = argsort(t0)
        if r30[0] > r30[-1]:
            ind = ind[::-1] # reverse
        t0, t1, lb, ub = t0[ind], t1[ind], lb[ind], ub[ind]
        lb, ub = hstack((y0, y0+(lb*(t1-t0)).cumsum())), hstack((y0, y0+(ub*(t1-t0)).cumsum()))
        #y_var = p._x0.keys()[0]
        #p.xf = p.xk = 0.5*(lb+ub)
        p.extras = {'startTimes': t0, 'endTimes': t1, eq_var:{'infinums': lb, 'supremums': ub}}
        return t0, t1, lb, ub
    elif isIP:
        P = p.point([nan]*p.n)
        P._f = F
        P._mr = ftol - r33
        P._mrName = 'None'
        P._mrInd = 0
#        p.xk = array([nan]*p.n)
#        p.rk = r33
#        p.fk = F
        #p._Residual = 
        p.iterfcn(asarray([nan]*p.n), fk=F, rk = ftol - r33)
    else:
        p.err('incorrect prob type in interalg ODE routine')



from wh_conn_solver import wh_conn_solver

class fmincon(wh_conn_solver):
    __name__ = 'fmincon'
    __authors__ = ""
    __alg__ = ""
    __optionalDataThatCanBeHandled__ = ['A', 'Aeq', 'b', 'beq', 'lb', 'ub', 'c', 'h']
    arrAttribs = ('x0', 'lb', 'ub', 'A', 'Aeq', 'b', 'beq', 'nc', 'nh')
    funcAttribs = ('f', 'df', 'c', 'dc', 'h', 'dh')
    solver_id = 1

    def __init__(self): 
        pass




"""
Example of solving Mini-Max Problem
max { (x0-15)^2+(x1-80)^2, (x1-15)^2 + (x2-8)^2, (x2-8)^2 + (x0-80)^2 } -> min
Currently nsmm is single OO solver available for MMP
It defines function F(x) = max_i {f[i](x)}
and solves NSP F(x) -> min using solver ralg.
It's very far from specialized solvers (like MATLAB fminimax),
but it's better than having nothing at all,
and allows using of nonsmooth and noisy funcs.
This solver is intended to be enhanced in future.
"""
from numpy import *
from openopt import *

f1 = lambda x: (x[0]-15)**2 + (x[1]-80)**2
f2 = lambda x: (x[1]-15)**2 + (x[2]-8)**2
f3 = lambda x: (x[2]-8)**2 + (x[0]-80)**2
f = [f1, f2, f3]

# you can define matrices as numpy array, matrix, Python lists or tuples

#box-bound constraints lb <= x <= ub
lb = [0]*3# i.e. [0,0,0]
ub = [15,  inf,  80]

# linear ineq constraints A*x <= b
A = mat('4 5 6; 80 8 15')
b = [100,  350]

# non-linear eq constraints Aeq*x = beq
Aeq = mat('15 8 80')
beq = 90

# non-lin ineq constraints c(x) <= 0
c1 = lambda x: x[0] + (x[1]/8) ** 2 - 15
c2 = lambda x: x[0] + (x[2]/80) ** 2 - 15
c = [c1, c2]
#or: c = lambda x: (x[0] + (x[1]/8) ** 2 - 15, x[0] + (x[2]/80) ** 2 - 15)

# non-lin eq constraints h(x) = 0
h = lambda x: x[0]+x[2]**2 - x[1]

x0 = [0, 1, 2]
p = MMP(f,  x0,  lb = lb,  ub = ub,   A=A,  b=b,   Aeq = Aeq,  beq = beq,  c=c,  h=h, xtol = 1e-6, ftol=1e-6)
#p = MMP(f,  x0, ftol=1e-8)
# optional, matplotlib is required:
#p.plot=1
r = p.solve('nsmm', iprint=1, NLPsolver = 'ralg', maxIter=1e3, minIter=1e2)
print 'MMP result:',  r.ff

#
### let's check result via comparison with NSP solution
F= lambda x: max([f1(x),  f2(x),  f3(x)])
p = NSP(F,  x0, lb = lb, ub = ub,  c=c,  h=h,  A=A,  b=b,  Aeq = Aeq,  beq = beq, xtol = 1e-6, ftol=1e-6)
#p = NSP(F,  x0)
r_nsp = p.solve('ralg')
#print 'NSP result:',  r_nsp.ff,  'difference:', r_nsp.ff - r.ff
#"""
#starting solver nsmm (license: BSD)  with problem  unnamed
#  iter       ObjFun        log10(maxResidual)
#     0       6.4660e+03   +1.89
#   10       6.4860e+03   -0.68
#   20       6.4158e+03   -1.23
#   30       6.4119e+03   -3.08
#   40       6.3783e+03   -2.95
#   50       6.3950e+03   -4.05
#   60       6.3951e+03   -6.02
#   70       6.3938e+03   -6.02
#   78       6.3936e+03   -6.00
#nsmm has finished solving the problem unnamed
#istop:  3 (|| X[k] - X[k-1] || < xtol)
#Solver:   Time Elapsed = 0.41   CPU Time Elapsed = 0.38
#objFunValue: 6393.6196095379446 (feasible, max constraint =  9.95421e-07)
#MMP result: 6393.6196095379446
#starting solver ralg (license: BSD)  with problem  unnamed
#itn 0 : Fk= 6466.0 MaxResidual= 78.0
#itn 10  Fk: 6485.9728487666425 MaxResidual: 2.07e-01 ls: 2
#itn 20  Fk: 6415.8358391383163 MaxResidual: 5.92e-02 ls: 1
#itn 30  Fk: 6411.9310394431113 MaxResidual: 8.22e-04 ls: 3
#itn 40  Fk: 6378.3471060481961 MaxResidual: 1.12e-03 ls: 2
#itn 50  Fk: 6394.9848936519056 MaxResidual: 8.94e-05 ls: 0
#itn 60  Fk: 6395.054402295913 MaxResidual: 9.57e-07 ls: 1
#itn 70  Fk: 6393.8314202292149 MaxResidual: 9.63e-07 ls: 1
#itn 78  Fk: 6393.6196095379446 MaxResidual: 9.95e-07 ls: 1
#ralg has finished solving the problem unnamed
#istop:  3 (|| X[k] - X[k-1] || < xtol)
#Solver:   Time Elapsed = 0.44   CPU Time Elapsed = 0.32
#objFunValue: 6393.6196095379446 (feasible, max constraint =  9.95421e-07)
#NSP result: 6393.6196095379446 difference: 0.0
#"""



from wh_conn_solver import wh_conn_solver

class matlab_fsolve(wh_conn_solver):
    __name__ = 'matlab_fsolve'
    __authors__ = ""
    __alg__ = ""
    __optionalDataThatCanBeHandled__ = ()
    arrAttribs = ('x0', )
    funcAttribs = ('f', 'df')
    solver_id = 2

    def __init__(self): 
        pass




from scipy.optimize import fmin_slsqp
import openopt
from openopt.kernel.setDefaultIterFuncs import *
from openopt.kernel.ooMisc import WholeRepr2LinConst, xBounds2Matrix
from openopt.kernel.baseSolver import baseSolver
from numpy import *

class EmptyClass: pass

class scipy_slsqp(baseSolver):
    __name__ = 'scipy_slsqp'
    __license__ = "BSD"
    __authors__ = """Dieter Kraft, connected to scipy by Rob Falck, connected to OO by Dmitrey"""
    __alg__ = "Sequential Least SQuares Programming"
    __info__ = 'constrained NLP solver'
    __optionalDataThatCanBeHandled__ = ['A', 'Aeq', 'b', 'beq', 'lb', 'ub', 'c', 'h']

    def __init__(self): pass
    def __solver__(self, p):
        bounds = []
        if any(isfinite(p.lb)) or any(isfinite(p.ub)):
            ind_inf = where(p.lb==-inf)[0]
            p.lb[ind_inf] = -1e50
            ind_inf = where(p.ub==inf)[0]
            p.ub[ind_inf] = 1e50
            for i in range(p.n):
                bounds.append((p.lb[i], p.ub[i]))
        empty_arr = array(())
        empty_arr_n = array(()).reshape(0, p.n)
        if not p.userProvided.c:
            p.c = lambda x: empty_arr.copy()
            p.dc = lambda x: empty_arr_n.copy()
        if not p.userProvided.h:
            p.h = lambda x: empty_arr.copy()        
            p.dh = lambda x: empty_arr_n.copy()
        C = lambda x: -hstack((p.c(x), p.matmult(p.A, x) - p.b))
        fprime_ieqcons = lambda x: -vstack((p.dc(x), p.A))
        #else: C,  fprime_ieqcons = None,  None
        #if p.userProvided.h:
        H = lambda x: hstack((p.h(x), p.matmult(p.Aeq, x) - p.beq))
        fprime_eqcons = lambda x: vstack((p.dh(x), p.Aeq))
        #else: H,  fprime_eqcons = None,  None
       # fprime_cons = lambda x: vstack((p.dh(x), p.Aeq, p.dc(x), p.A))

        x, fx, its, imode, smode = fmin_slsqp(p.f, p.x0, bounds=bounds,  f_eqcons = H, f_ieqcons = C, full_output=1, iprint=-1, fprime = p.df, fprime_eqcons = fprime_eqcons, fprime_ieqcons = fprime_ieqcons, acc = p.contol, iter = p.maxIter)
        p.msg = smode
        if imode == 0: p.istop = 1000
        #elif imode == 9: p.istop = ? CHECKME that OO kernel is capable of handling the case
        else: p.istop = -1000
        p.xk, p.fk = array(x), fx




PythonSum = sum
from numpy import isnan, array, atleast_1d, asarray, logical_and, all, logical_or, any, \
arange, vstack, inf, logical_not, take, abs, empty, \
isfinite, argsort, ones, zeros, log1p, array_split#where

# for PyPy
from openopt.kernel.nonOptMisc import where

#try:
#    from bottleneck import nanargmin, nanmin
#except ImportError:
#    from numpy import nanmin, nanargmin
from interalgLLR import *

def r43_seq(Arg):
    targets_vals, targets_tols, solutionsF, lf, uf = Arg
    lf, uf = asarray(lf), asarray(uf)
    if lf.size == 0 or len(solutionsF) == 0: return None

    m = lf.shape[0]
    n = lf.shape[2]/2
    r = zeros((m, 2*n))

    for _s in solutionsF:
        s = atleast_1d(_s)
        tmp = ones((m, 2*n))
        for i in range(len(targets_vals)):
            val, tol = targets_vals[i], targets_tols[i]#, t.val, t.tol
            #TODO: mb optimize it
            o, a = lf[:, i], uf[:, i] 
            if val == inf:
                ff = s[i] + tol
                ind = a > ff
                if any(ind):
                    t1 = a[ind]
                    t2 = o[ind]
                    t_diff = t1-t2
                    t_diff[t_diff<1e-200] = 1e-200
                    # TODO: check discrete cases
                    Tmp = (ff-t2) / t_diff

                    tmp[ind] *= Tmp
                    tmp[ff<o] = 0.0
                    
            elif val == -inf:
                ff = s[i] - tol
                ind = o < ff
                if any(ind):
                    t1 = a[ind]
                    t2 = o[ind]
                    t_diff = t1-t2
                    t_diff[t_diff<1e-200] = 1e-200
                    
                    Tmp = (t1-ff) / t_diff

                    tmp[ind] *= Tmp
                    tmp[a<ff] = 0.0
            else: # finite val
                ff = abs(s[i]-val) - tol
                if ff <= 0:
                    continue
                _lf, _uf = o - val, a - val
                ind = logical_or(_lf < ff, _uf > - ff)
                _lf = _lf[ind]
                _uf = _uf[ind]
                _lf[_lf>ff] = ff
                _lf[_lf<-ff] = -ff
                _uf[_uf<-ff] = -ff
                _uf[_uf>ff] = ff
                
                r20 = a[ind] - o[ind]
                r20[r20<1e-200] = 1e-200
                _diff = _uf - _lf
                _diff[_diff<1e-200] = 1e-200
                
                Tmp = 1.0 - (_uf - _lf) / r20
                
                tmp[ind] *= Tmp

        new = 0
        if new:
            ind_0 = tmp == 0.0
            ind_1 = tmp == 1.0
            r[ind_1] = inf
            ind_m = logical_not(logical_and(ind_0, ind_1))
            #r[ind_m] -= log1p(-tmp[ind_m]) * 1.4426950408889634
            r[ind_m] -= log1p(-tmp[ind_m]) * 1.4426950408889634
            #r20 = log2(1-tmp[ind_m]) #* 1.4426950408889634
            
        else:
            r -= log1p(-tmp) * 1.4426950408889634 # log2(e)
            
        #r -= r20
    return r

from multiprocessing import Pool
def r43(targets, SolutionsF, lf, uf, pool, nProc):
    lf, uf = asarray(lf), asarray(uf)
    target_vals = [t.val for t in targets]
    target_tols = [t.tol for t in targets]
    if nProc == 1 or len(SolutionsF) <= 1:
        return r43_seq((target_vals, target_tols, SolutionsF, lf, uf))
        
    splitBySolutions = True #if len(SolutionsF) > max((4*nProc, ))
    if splitBySolutions:
        ss = array_split(SolutionsF, nProc)
        Args = [(target_vals, target_tols, s, lf, uf) for s in ss]
        result = pool.imap_unordered(r43_seq, Args)#, callback = cb)    
        r = [elem for elem in result if elem is not None]
        return PythonSum(r)
    else:
        lf2 = array_split(lf, nProc)
        uf2 = array_split(uf, nProc)
        Args = [(target_vals, target_tols, SolutionsF, lf2[i], uf2[i]) for i in range(nProc)]
        result = pool.map(r43_seq, Args)
        r = [elem for elem in result if elem is not None]
        return vstack(r)

def r14MOP(p, nlhc, residual, definiteRange, y, e, vv, asdf1, C, r40, g, nNodes,  \
         r41, fTol, Solutions, varTols, _in, dataType, \
         maxNodes, _s, indTC, xRecord):

    assert p.probType == 'MOP'
    
#    if len(p._discreteVarsNumList):
#        y, e, ind_trunc = adjustDiscreteVarBounds(y, e, p)
#        if ind_trunc.size:
#            
#            _s, indTC, 
    
    
    if p.nProc != 1 and getattr(p, 'pool', None) is None:
        p.pool = Pool(processes = p.nProc)
    elif p.nProc == 1:
        p.pool = None
    
    ol, al = [], []
    targets = p.targets # TODO: check it
    m, n = y.shape
    ol, al = [[] for k in range(m)], [[] for k in range(m)]
    for i, t in enumerate(targets):
        y, e, o, a, definiteRange, exactRange, _s_, indTC_ = func82(y, e, vv, t.func, dataType, p)
#        o, a, definiteRange, exactRange, _s, indTC = func82(y, e, vv, t.func, dataType, p, _s, indTC)
        o, a = o.reshape(2*n, m).T, a.reshape(2*n, m).T
        for j in range(m):
            ol[j].append(o[j])
            al[j].append(a[j])
        #ol.append(o.reshape(2*n, m).T.tolist())
        #al.append(a.reshape(2*n, m).T.tolist())

    nlhf = r43(targets, Solutions.F, ol, al, p.pool, p.nProc)
    
    fo_prev = 0
    # TODO: remove NaN nodes here

    if y.size == 0:
        return _in, g, fo_prev, _s, Solutions, xRecord, r41, r40
    
    nodes = func11(y, e, nlhc, indTC, residual, ol, al, _s, p)
    
    #y, e = func4(y, e, o, a, fo)
    
    
    assert p.solver.dataHandling == 'raw', '"sorted" mode is unimplemented for MOP yet'
    
    if nlhf is None:
        new_nodes_tnlh_all = nlhc
    elif nlhc is None: 
        new_nodes_tnlh_all = nlhf
    else:
        new_nodes_tnlh_all = nlhf + nlhc

    asdf1 = [t.func for t in p.targets]
    r5F, r5Coords = getr4Values(vv, y, e, new_nodes_tnlh_all, asdf1, C, p.contol, dataType, p) 
    
    nIncome, nOutcome = r44(Solutions, r5Coords, r5F, targets, p.solver.sigma)
    fo = 0 # unused for MOP
    
    # TODO: better of nlhc for unconstrained probs

#    if len(_in) != 0:
#        an = hstack((nodes,  _in))
#    else:
#        an = atleast_1d(nodes)

    an = nodes + _in

    hasNewParetoNodes = False if nIncome == 0 else True
    if hasNewParetoNodes:
        ol2 = [node.o for node in an]
        al2 = [node.a for node in an]
        nlhc2 = [node.nlhc for node in an]
        nlhf2 = r43(targets, Solutions.F, ol2, al2, p.pool, p.nProc)
        tnlh_all = asarray(nlhc2) if nlhf2 is None else nlhf2 if nlhc2[0] is None else asarray(nlhc2) + nlhf2
    else:
        tnlh_all = vstack([new_nodes_tnlh_all] + [node.tnlh_all for node in _in]) if len(_in) != 0 else new_nodes_tnlh_all
        
    for i, node in enumerate(nodes):
        node.tnlh_all = tnlh_all[i]

    r10 = logical_not(any(isfinite(tnlh_all), 1))
    if any(r10):
        ind = where(logical_not(r10))[0]
#        an = asarray(an[ind])
        an = [an[i] for i in ind]
        tnlh_all = take(tnlh_all, ind, axis=0, out=tnlh_all[:ind.size])
    
#    else:
#        tnlh_all = hstack([node.tnlh_all for node in an])
        
    T1, T2 = tnlh_all[:, :tnlh_all.shape[1]/2], tnlh_all[:, tnlh_all.shape[1]/2:]
    T = where(logical_or(T1 < T2, isnan(T2)), T1, T2)
    t = nanargmin(T, 1)
    
    w = arange(t.size)
    NN = T[w, t].flatten()

    for i, node in enumerate(an):
        node.tnlh_all = tnlh_all[i]
        node.tnlh_curr_best = NN[i]
        
    astnlh = argsort(NN)
    
#    an = an[astnlh]
    an = [an[i] for i in astnlh]

    p._t = t
    
    # TODO: form _s in other level (for active nodes only), to reduce calculations
    if len(an) != 0:
        T = asarray([node.nlhf for node in an])
#        nlhc_fixed = asarray([node.nlhc for node in an])
        if an[0].nlhc is not None:
            T += asarray([node.nlhc for node in an])
#        T = nlhf_fixed + nlhc_fixed if nlhc_fixed[0] is not None else nlhf_fixed 
        p.__s = \
        nanmin(vstack(([T[w, t], T[w, n+t]])), 0)
    else:
        p.__s = array([])

#        p._nObtainedSolutions = len(solutions)
#        if p._nObtainedSolutions > maxSolutions:
#            solutions = solutions[:maxSolutions]
#            p.istop = 0
#            p.msg = 'user-defined maximal number of solutions (p.maxSolutions = %d) has been exeeded' % p.maxSolutions
#            return an, g, fo, None, solutions, coords, xRecord, r41, r40
    
    
    # TODO: fix it
    p._frontLength = len(Solutions.F)
    p._nIncome = nIncome
    p._nOutcome = nOutcome
    p.iterfcn(p.x0)
    #print('iter: %d (%d) frontLenght: %d' %(p.iter, itn, len(Solutions.coords)))
    
    if p.istop != 0: 
        return an, g, fo, None, Solutions, xRecord, r41, r40
        
    #an, g = func9(an, fo, g, p)

    nn = maxNodes#1 if asdf1.isUncycled and all(isfinite(o)) and p._isOnlyBoxBounded and not p.probType.startswith('MI') else maxNodes
    
    an, g = func5(an, nn, g, p)
    nNodes.append(len(an))
    return an, g, fo, _s, Solutions, xRecord, r41, r40



def r44(Solutions, r5Coords, r5F, targets, sigma):
#    print Solutions.F
#    if len(Solutions.F) != Solutions.coords.shape[0]:
#        raise 0
    # TODO: rework it
    #sf = asarray(Solutions.F)
    nIncome, nOutcome = 0, 0
    m= len(r5Coords)
    #n = len(r5Coords[0])
    # TODO: mb use inplace r5Coords / r5F modification instead?
    for j in range(m):
        if isnan(r5F[0][0]):
            continue
        if Solutions.coords.size == 0:
            Solutions.coords = array(r5Coords[j]).reshape(1, -1)
            Solutions.F.append(r5F[0])
            nIncome += 1
            continue
        M = Solutions.coords.shape[0] 
        
        r47 = empty(M, bool)
        r47.fill(False)
#        r48 = empty(M, bool)
#        r48.fill(False)
        for i, target in enumerate(targets):
            
            f = r5F[j][i]
            
            # TODO: rewrite it
            F = asarray([Solutions.F[k][i] for k in range(M)])
            #d = f - F # vector-matrix
            
            val, tol = target.val, target.tol
            Tol = sigma * tol
            if val == inf:
                r52 = f > F + Tol
#                r36olution_better = f <= F#-tol
            elif val == -inf:
                r52 = f < F - Tol
#                r36olution_better = f >= F#tol
            else:
                r52 = abs(f - val) < abs(F - val) - Tol
#                r36olution_better = abs(f - val) >= abs(Solutions.F[i] - val)#-tol # abs(Solutions.F[i] - target)  < abs(f[i] - target) + tol
            
            r47 = logical_or(r47, r52)
#            r48 = logical_or(r48, r36olution_better)
        
        accept_c = all(r47)
        #print sum(asarray(Solutions.F))/asarray(Solutions.F).size
        if accept_c:
            nIncome += 1
            #new
            r48 = empty(M, bool)
            r48.fill(False)
            for i, target in enumerate(targets):
                f = r5F[j][i]
                F = asarray([Solutions.F[k][i] for k in range(M)])
                val, tol = target.val, target.tol
                if val == inf:
                    r36olution_better = f < F
                elif val == -inf:
                    r36olution_better = f > F
                else:
                    r36olution_better = abs(f - val) > abs(F - val)
                r48 = logical_or(r48, r36olution_better)

            r49 = logical_not(r48)
            remove_s = any(r49)
            if remove_s:
                r50 = where(r49)[0]
                nOutcome += r50.size
                Solutions.coords[r50[0]] = r5Coords[j]
                Solutions.F[r50[0]] = r5F[j]
                
                if r50.size > 1:
                    r49[r50[0]] = False
                    indLeft = logical_not(r49)
                    indLeftPositions = where(indLeft)[0]
                    newSolNumber = Solutions.coords.shape[0] - r50.size + 1
                    Solutions.coords = take(Solutions.coords, indLeftPositions, axis=0, out = Solutions.coords[:newSolNumber])
                    Solutions.F = [Solutions.F[i] for i in indLeftPositions]
            else:
                Solutions.coords = vstack((Solutions.coords, r5Coords[j]))
                Solutions.F.append(r5F[j])
    return nIncome, nOutcome



from NLOPT_AUX import NLOPT_AUX
from NLOPT_BASE import NLOPT_BASE
import nlopt

class isres(NLOPT_BASE):
    __name__ = 'isres'
    __alg__ = 'Improved Stochastic Ranking Evolution Strategy'
    __authors__ = 'S. G. Johnson'
    
    __optionalDataThatCanBeHandled__ = ['lb', 'ub', 'A', 'Aeq', 'b', 'beq', 'c', 'h']
    funcForIterFcnConnection = 'f'
    population = 0
    #__isIterPointAlwaysFeasible__ = lambda self, p: True
    __isIterPointAlwaysFeasible__ = True
    _requiresFiniteBoxBounds = True
    
    def __init__(self): pass
    def __solver__(self, p):
        nlopt_opts = {'set_population':self.population} if self.population != 0 else {}
        if self.population != 0: p.f_iter = self.population
        NLOPT_AUX(p, nlopt.GN_ISRES, nlopt_opts)



from numpy import cos, arange, ones, asarray, zeros, mat, array
from openopt import NLP

def test(complexity=0, **kwargs):
    n = 15 * (complexity+1)

    x0 = 15*cos(arange(n)) + 8

    f = lambda x: ((x-15)**2).sum()
    df = lambda x: 2*(x-15)

    c = lambda x: [2* x[0] **4-32, x[1]**2+x[2]**2 - 8]

    # dc(x)/dx: non-lin ineq constraints gradients (optional):
    def dc(x):
        r = zeros((len(c(x0)), n))
        r[0,0] = 2 * 4 * x[0]**3
        r[1,1] = 2 * x[1]
        r[1,2] = 2 * x[2]
        return r

    hp = 2
    h1 = lambda x: (x[-1]-13)**hp
    h2 = lambda x: (x[-2]-17)**hp
    h = lambda x:[h1(x), h2(x)]

    # dh(x)/dx: non-lin eq constraints gradients (optional):
    def dh(x):
        r = zeros((2, n))
        r[0, -1] = hp*(x[-1]-13)**(hp-1)
        r[1, -2] = hp*(x[-2]-17)**(hp-1)
        return r

    lb = -8*ones(n)
    ub = 15*ones(n)+8*cos(arange(n))

    ind = 3

    A = zeros((2, n))
    A[0, ind:ind+2] = 1
    A[1, ind+2:ind+4] = 1
    b = [15,  8]

    Aeq = zeros(n)
    Aeq[ind+4:ind+8] = 1
    beq = 45
    ########################################################
    colors = ['b', 'k', 'y', 'g', 'r']
    #solvers = ['ipopt', 'ralg','scipy_cobyla']
    solvers = ['ralg','scipy_slsqp', 'ipopt']
    solvers = [ 'ralg', 'scipy_slsqp']
    solvers = [ 'ralg']
    solvers = [ 'r2']
    solvers = [ 'ralg', 'scipy_slsqp']
    ########################################################
    for i, solver in enumerate(solvers):
        p = NLP(f, x0, df=df, c=c, h=h, dc=dc, dh=dh, lb=lb, ub=ub, A=A, b=b, Aeq=Aeq, beq=beq, maxIter = 1e4, \
                show = solver==solvers[-1], color=colors[i],  **kwargs )
        if not kwargs.has_key('iprint'): p.iprint = -1
#        p.checkdf()
#        p.checkdc()
#        p.checkdh()
        r = p.solve(solver)
    if r.istop>0: return True, r, p
    else: return False, r, p

if __name__ == '__main__':
    isPassed, r, p = test(iprint= 10, plot=0)
    assert isPassed
